/*
   Copyright (c) 2010 - 2013 Javaika.com by Alexander Wait.
   
   All Rights Reserved.
*/



package main;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.InetAddress;
import java.net.URL;
import java.util.ArrayList;

import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.AbstractDocument;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DocumentFilter;

import required.AudioVideoProcessor;
import required.AudioVideoReceive;
import required.AudioVideoTransmit;
import required.ColorFunctions;
import required.DataUploadDownload;
import required.Domain;
import required.Message;
import required.Portal;
import required.Validation;


//********************************************* START CLASS *********************************************//


/**
 * This class builds the online messaging GUI.
 * 
 * @author Alexander Wait
 * @version 2014-07-25
 */
public class OnlineMessagingGui extends JApplet
{
   private static final String copyright = "Copyright (c) 2012 by Alexander Wait - All Rights Reserved";

   private static final long serialVersionUID = 1L;
   

   public static boolean isApplication = false;
   
   public static boolean runRestricted = false;
   
   public static boolean isVisible = false;
   

   /* Interface Settings */

   private static final int FRAME_SIZE_X = 520;
   
   private static final int FRAME_SIZE_Y = 590;
   
   private static final int APPLET_SIZE_Y = 560;
   
   
   /* Set Poll Settings */

   private static final int PING_TIMEOUT = 3000;
   
   
   /* Validation Settings */
   
   private static final int MAX_PORT_LENGTH = 5;
   
   private static final int MAX_VIDEO_ID_LENGTH = 8;
   
   private static final int MIN_CHANNEL_ID_LENGTH = 6;
   
   private static final int MAX_CHANNEL_ID_LENGTH = 20;
   

   private static final String MESSAGE_CLEAR_ALL = "http://" + Domain.getDomain()
   
   + "/bridge/record-message-clear-all.php" + Domain.getPort(); // Port:80

   
   private static final String MESSAGE_CLEAR_ID = "http://" + Domain.getDomain() 
   
   + "/bridge/record-message-clear-id.php" + Domain.getPort(); // Port:80
   
   
   //************************************** START INITIALISATION CODE **************************************//

   
   /**
    * Browser constructor.
    */
   public void init()
   {  
      String copyrightParam = getParameter("copyright");
       
      if ((copyrightParam == null) || !copyrightParam.equals(copyright)) 
      {
         Message.showMessage("Invalid Copyright", "WARNING", "warning"); 
      }       
       
      else if (!getDocumentBase().getHost().equals(Domain.getDomain()))
      {
         Message.showMessage("Unauthorised Applet Use", "ERROR", "error");
      }

      else
      {
         try {getToolkit().getSystemClipboard();}
          
         catch (java.security.AccessControlException e) {runRestricted = true;} 
 

         userId = getParameter("id"); userPw = getParameter("pw");

         
         initialiser(); includeResizeEvent(); setJMenuBar(menuBar); 
         
         setSize(FRAME_SIZE_X,FRAME_SIZE_Y);
      }
   }
   
   
   
   /**
    * Application constructor.
    */
   public OnlineMessagingGui()
   {
      if (isApplication)
      { 
         initialiser();  
      
      
         frame = new JFrame();

         frame.setMinimumSize(new Dimension(FRAME_SIZE_X, FRAME_SIZE_Y));
         
         frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

         frame.setSize(FRAME_SIZE_X, FRAME_SIZE_Y);

         frame.setTitle("Transceiver Program");
      
         frame.setLocationRelativeTo(null); 
      
         frame.setJMenuBar(menuBar);

         frame.add(contentPane);  
      
      
         setVisibility();
      }
      
   }
   
   
   
   /**
    * GUI initializer.
    * ESCA-JAVA0076:
    */
   private void initialiser()
   {
      validation = new Validation();
      
      filter = new UppercaseDocumentFilter();
      

      // Initialize content pane and layout.

      contentPane = getContentPane();
      
      contentPane.setLayout(new GridBagLayout());
      
      
      // Initialize JLabel video window.
      
      label = new JLabel();
      
      label.setBorder(BorderFactory.createLineBorder(new Color(240, 0, 0)));
      
      label.setIcon(new ImageIcon(this.getClass().getResource
  
      ("/files/images/VideoMessagingImageBlank.png")));
      
      
      // Initialize button GUI components.
      
      start = new JButton(new ImageIcon(this.getClass().getResource
      (
         "/files/images/TransmitReceiveStartNormal.png"))
      ); 
      
      
      start.setRolloverIcon(new ImageIcon(this.getClass().getResource
      (
         "/files/images/TransmitReceiveStartRollover.png"))
      );

      
      start.setPressedIcon(new ImageIcon(this.getClass().getResource
      (
         "/files/images/TransmitReceiveStartPressed.png"))
      );
      
      
      start.setBorder(BorderFactory.createEmptyBorder(0,0,0,0));
      
      start.setPreferredSize(new Dimension(115, 25)); 
      
      start.addActionListener(new StartButtonListener());
      
      
      stop = new JButton(new ImageIcon(this.getClass().getResource
      (
         "/files/images/TransmitReceiveStopNormal.png"))
      ); 
      
      
      stop.setRolloverIcon(new ImageIcon(this.getClass().getResource
      (
         "/files/images/TransmitReceiveStopRollover.png"))
      );

      
      stop.setPressedIcon(new ImageIcon(this.getClass().getResource
      (
         "/files/images/TransmitReceiveStopPressed.png"))
      );
      
      
      stop.setBorder(BorderFactory.createEmptyBorder(0,0,0,0));
      
      stop.setPreferredSize(new Dimension(115, 25)); 
      
      stop.addActionListener(new StopButtonListener());
      
      
      delete = new JButton("Delete"); delete.setFocusPainted(false); 
      
      delete.setPreferredSize(new Dimension(70, 30));
      
      delete.addActionListener(new DeleteMessageListener());
      
      
      // Initialize text field GUI components.
      
      videoId = new JTextField(); videoId.setText("1");
      
      videoId.setPreferredSize(new Dimension(93, 60));
      
      videoId.setHorizontalAlignment(JTextField.CENTER);
      
      videoId.setBorder(BorderFactory.createTitledBorder("Video ID"));
      
      
      channelId = new JTextField();
      
      channelId.setPreferredSize(new Dimension(93, 60));
      
      channelId.setHorizontalAlignment(JTextField.CENTER);

      channelId.setBorder(BorderFactory.createTitledBorder("Channel ID"));
      
      ((AbstractDocument) channelId.getDocument()).setDocumentFilter(filter);
      
      
      deleteId = new JTextField(); deleteId.setText("1");
      
      deleteId.setPreferredSize(new Dimension(93, 60));
      
      deleteId.setHorizontalAlignment(JTextField.CENTER);
      
      deleteId.setBorder(BorderFactory.createTitledBorder("Video ID"));
      
      
      ipAddress = new JTextField(); ipAddress.setText("127.0.0.1");
      
      ipAddress.setPreferredSize(new Dimension(97, 60));
      
      ipAddress.setHorizontalAlignment(JTextField.CENTER);
      
      ipAddress.setBorder(BorderFactory.createTitledBorder("Address"));
      
      
      dataTransmitPorts = new JTextField(); dataTransmitPorts.setText("300:800");
      
      dataTransmitPorts.setPreferredSize(new Dimension(70, 60));
      
      dataTransmitPorts.setHorizontalAlignment(JTextField.CENTER);
      
      dataTransmitPorts.setBorder(BorderFactory.createTitledBorder("Ports"));
      
      
      // Initialize radio button GUI components.

      mono = new JRadioButton("1", true); mono.setFocusPainted(false); 

      mono.setSelected(true); 
      
      
      stereo = new JRadioButton("2", true); stereo.setFocusPainted(false); 
      
      stereo.setSelected(false);
      
      
      minBytes = new JRadioButton("1", true); minBytes.setFocusPainted(false); 
      
      minBytes.setSelected(false);
      
      
      maxBytes = new JRadioButton("2", true); maxBytes.setFocusPainted(false); 
      
      maxBytes.setSelected(true);
      
      
      // Initialize button group GUI components.
      
      channelsGroup = new ButtonGroup(); 
      
      channelsGroup.add(mono); channelsGroup.add(stereo);
      
      
      bytesGroup = new ButtonGroup(); 
      
      bytesGroup.add(minBytes); bytesGroup.add(maxBytes); 
 
      
      // Initialize combo box GUI components.
      
      protocol = new JComboBox<String>(protocolChoices);
      
      protocol.setPreferredSize(new Dimension(50, 19));
      
      protocol.setMaximumRowCount(2); protocol.setSelectedIndex(0);
      
      
      camera = new JComboBox<String>(cameraChoices);
      
      camera.setPreferredSize(new Dimension(50, 19));
      
      camera.setMaximumRowCount(2); camera.setSelectedIndex(0);
      

      samples = new JComboBox<String>(samplesChoices);
      
      samples.setPreferredSize(new Dimension(65, 19));
      
      samples.setMaximumRowCount(2); samples.setSelectedIndex(1);
      
      
      size = new JComboBox<String>(sizeChoices);
      
      size.setPreferredSize(new Dimension(80, 19));
      
      size.setMaximumRowCount(2); size.setSelectedIndex(2);

      
      // Initialize check box GUI components.
      
      transmit = new JCheckBox(); transmit.setFocusPainted(false);

      transmit.addChangeListener(new SelectListener(0));
      
      transmit.setSelected(true);

      
      receive = new JCheckBox(); receive.setFocusPainted(false);
      
      receive.addChangeListener(new SelectListener(1));
      
      receive.setSelected(true);
      
      
      deleteAll = new JCheckBox(); deleteAll.setFocusPainted(false);

      deleteAll.addChangeListener(new SelectListener(0));
      
      deleteAll.setSelected(true);
      
      
      record = new JCheckBox(); record.setFocusPainted(false);
      
      record.setSelected(true);
      
      
      play = new JCheckBox(); play.setFocusPainted(false);
     
      video = new JCheckBox(); video.setFocusPainted(false); video.setSelected(true);
      
      
      ButtonGroup group = new ButtonGroup(); 
      
      group.add(record); group.add(play);
      
      
      // Initialize slider GUI components.
      
      rate = new JSlider();
      
      rate.setPreferredSize(new Dimension(60, 30));
      
      rate.setMajorTickSpacing(25); rate.setValue(75);

      rate.setPaintTicks(true);
      

      quality = new JSlider();
      
      quality.setPreferredSize(new Dimension(60, 30));
      
      quality.setMajorTickSpacing(25); quality.setValue(50);

      quality.setPaintTicks(true);
      
      
      chunks = new JSlider();
      
      chunks.setPreferredSize(new Dimension(60, 30));   
      
      chunks.setMajorTickSpacing(25); chunks.setValue(50);

      chunks.setPaintTicks(true);

      
      // Initialize progress bar GUI components.
      
      progressBar = new JProgressBar();  
      
      progressBar.setPreferredSize(new Dimension(180, 25));
      
      progressBar.setStringPainted(true); progressBar.setString("Pending");   
      
      
      // Initialize text area GUI components.
      
      textOutput = new JTextArea(); textOutput.setEditable(false);
      
      textOutput.setFont(new Font("Verdana", Font.ITALIC, 11)); 
      
      textOutput.setForeground(Color.GRAY);
      
      
      textPane = new JScrollPane(textOutput, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS, 

      JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);


      textPane.setBorder(BorderFactory.createEmptyBorder(13, 10, 13, 10));
      

      // Initialize JPanel GUI components.
      
      imagePanel = new JPanel(); imagePanel.setLayout(new FlowLayout());
      
      imagePanel.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
      
      imagePanel.add(label);
      
      
      imagePane = new JScrollPane(imagePanel, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
      
      JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
      
      
      imagePane.setPreferredSize(new Dimension(465, 325));
      
      
      transmitPanel = new JPanel(); transmitPanel.add(transmit);
      
      transmitPanel.setPreferredSize(new Dimension(70, 60));

      transmitPanel.setBorder(BorderFactory.createTitledBorder("Transmit"));
      
      
      receivePanel = new JPanel(); receivePanel.add(receive);
      
      receivePanel.setPreferredSize(new Dimension(70, 60));

      receivePanel.setBorder(BorderFactory.createTitledBorder("Receive"));
      
      
      protocolPanel = new JPanel(); protocolPanel.add(protocol);
      
      protocolPanel.setPreferredSize(new Dimension(70, 60));

      protocolPanel.setBorder(BorderFactory.createTitledBorder("Protocol"));
      
      
      deleteAllPanel = new JPanel(); deleteAllPanel.add(deleteAll);
      
      deleteAllPanel.setPreferredSize(new Dimension(93, 60));

      deleteAllPanel.setBorder(BorderFactory.createTitledBorder("Delete All"));
      

      recordPanel = new JPanel(); recordPanel.add(record);
      
      recordPanel.setPreferredSize(new Dimension(93, 60));

      recordPanel.setBorder(BorderFactory.createTitledBorder("Record"));
      
      
      playPanel = new JPanel(); playPanel.add(play);
      
      playPanel.setPreferredSize(new Dimension(93, 60));

      playPanel.setBorder(BorderFactory.createTitledBorder("Play"));
      
      
      cameraPanel = new JPanel(); cameraPanel.add(camera);
      
      cameraPanel.setPreferredSize(new Dimension(70,60));
      
      cameraPanel.setBorder(BorderFactory.createTitledBorder("Camera"));
      
      
      samplesPanel = new JPanel(); samplesPanel.add(samples);
      
      samplesPanel.setPreferredSize(new Dimension(90,60));
      
      samplesPanel.setBorder(BorderFactory.createTitledBorder("Samples"));
      
      
      channelsPanel = new JPanel(); channelsPanel.add(mono); channelsPanel.add(stereo);
      
      channelsPanel.setPreferredSize(new Dimension(100,60));
      
      channelsPanel.setBorder(BorderFactory.createTitledBorder("Channels"));
      
      
      bytesPanel = new JPanel(); bytesPanel.add(minBytes); bytesPanel.add(maxBytes);
      
      bytesPanel.setPreferredSize(new Dimension(100,60));
      
      bytesPanel.setBorder(BorderFactory.createTitledBorder("Bytes"));
      
      
      videoIdPanel = new JPanel(); videoIdPanel.add(videoId);
      
      videoIdPanel.setPreferredSize(new Dimension(97, 60));

      videoIdPanel.setBorder(BorderFactory.createTitledBorder("Video ID"));
      
      
      videoIncPanel = new JPanel(); videoIncPanel.add(video);
      
      videoIncPanel.setPreferredSize(new Dimension(97, 60));

      videoIncPanel.setBorder(BorderFactory.createTitledBorder("Include Video"));
      
      
      frameRatePanel = new JPanel(); frameRatePanel.add(rate);
      
      frameRatePanel.setPreferredSize(new Dimension(85, 60));

      frameRatePanel.setBorder(BorderFactory.createTitledBorder("Frame Rate"));
      
      
      videoQualityPanel = new JPanel(); videoQualityPanel.add(quality);
      
      videoQualityPanel.setPreferredSize(new Dimension(70, 60));

      videoQualityPanel.setBorder(BorderFactory.createTitledBorder("Quality"));
      
      
      chunkSizePanel = new JPanel(); chunkSizePanel.add(chunks);
      
      chunkSizePanel.setPreferredSize(new Dimension(97, 60));

      chunkSizePanel.setBorder(BorderFactory.createTitledBorder("Chunk Size"));
      
      
      videoSizePanel = new JPanel(); videoSizePanel.add(size);
      
      videoSizePanel.setPreferredSize(new Dimension(97,60));
      
      videoSizePanel.setBorder(BorderFactory.createTitledBorder("Video Size"));
      

      transceiverPanel = new JPanel(); transceiverPanel.setLayout(new FlowLayout());
      
      transceiverPanel.add(transmitPanel); transceiverPanel.add(receivePanel);
      
      transceiverPanel.add(ipAddress); transceiverPanel.add(dataTransmitPorts);
      
      transceiverPanel.add(protocolPanel);
      
     
      messagingPanel = new JPanel(); messagingPanel.setLayout(new FlowLayout());
      
      messagingPanel.add(recordPanel); messagingPanel.add(playPanel); 
      
      messagingPanel.add(videoId); messagingPanel.add(channelId); 
      
      
      deletePanel = new JPanel(); deletePanel.setLayout(new FlowLayout());
      
      deletePanel.add(deleteAllPanel); deletePanel.add(deleteId); deletePanel.add(delete);
      
      
      videoPanel = new JPanel(); videoPanel.setLayout(new FlowLayout());
      
      videoPanel.add(videoIncPanel); videoPanel.add(frameRatePanel);
      
      videoPanel.add(videoSizePanel); videoPanel.add(cameraPanel);
      
      videoPanel.add(videoQualityPanel);
      
      
      audioPanel = new JPanel(); videoPanel.setLayout(new FlowLayout());
      
      audioPanel.add(channelsPanel); audioPanel.add(bytesPanel); 
      
      audioPanel.add(chunkSizePanel); audioPanel.add(samplesPanel);
      
      
      feedbackPanel = new JPanel(); feedbackPanel.setLayout(new GridLayout(1, 1));
      
      feedbackPanel.add(textPane);
      
     
      buttonPanel = new JPanel(); 
      
      buttonPanel.setLayout (new FlowLayout(FlowLayout.LEADING));
      
      buttonPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
      
      
      // Initialize tab pane GUI components.
      
      tabbedPane = new JTabbedPane(); 
      
      
      tabbedPane.setFocusable(false);
      
      tabbedPane.setPreferredSize(new Dimension(450, 115));
      
      tabbedPane.addTab("Transceiver", transceiverPanel); 
      
      tabbedPane.addTab("Messaging", messagingPanel);
      
      tabbedPane.addTab("Delete", deletePanel);
      
      tabbedPane.addTab("Video", videoPanel);
      
      tabbedPane.addTab("Audio", audioPanel);
      
      tabbedPane.addTab("Feedback", feedbackPanel);
      
      tabbedPane.setSelectedIndex(0);
      

      // Add GUI components to JPanels.
      
      buttonPanel.add(progressBar);
      
      buttonPanel.add(start); buttonPanel.add(stop); 
 
     
      // Initialize menu bar GUI components.
      
      menuBar = new JMenuBar(); menuBar.add(createSettingsMenu());
      
      
      // Method calls for GUI initialization.
      
      gridBagConstraints(); setColors(); setGraphicIcons();
   }
   
   
   
   /**
    * Sets the background color.
    */
   private void setColors()
   {
      contentPane.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      label.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      textOutput.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      transmit.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      receive.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      video.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );

      record.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );

      mono.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      stereo.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      chunks.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      play.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      rate.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      quality.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      minBytes.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      maxBytes.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      channelId.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      videoId.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      ipAddress.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      deleteId.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      deleteAll.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );

      dataTransmitPorts.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      tabbedPane.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      progressBar.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      textPane.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      imagePanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      imagePane.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      cameraPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );

      samplesPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      protocolPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );

      buttonPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      recordPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      playPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      channelsPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
         
      bytesPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      bytesPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      transceiverPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      messagingPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      videoPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      audioPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      deletePanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      feedbackPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      transmitPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      receivePanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      videoIncPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      videoSizePanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      deleteAllPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      chunkSizePanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      frameRatePanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
      videoQualityPanel.setBackground
      (
         new Color (ColorFunctions.red(), ColorFunctions.green(), ColorFunctions.blue())
      );
      
   }
   
   
   
   /**
    * Sets icons for graphical elements.
    */
   private void setGraphicIcons()
   {
      mono.setIcon(new ImageIcon
      (this.getClass().getResource("/files/images/RadioButtonNormalUnselected.png")));
            
      mono.setSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/RadioButtonNormalSelected.png")));

      mono.setDisabledIcon(new ImageIcon
      (this.getClass().getResource("/files/images/RadioButtonDisabledUnselected.png")));

      mono.setDisabledSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/RadioButtonDisabledSelected.png")));
         
         
      stereo.setIcon(new ImageIcon
      (this.getClass().getResource("/files/images/RadioButtonNormalUnselected.png")));
                     
      stereo.setSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/RadioButtonNormalSelected.png")));

      stereo.setDisabledIcon(new ImageIcon
      (this.getClass().getResource("/files/images/RadioButtonDisabledUnselected.png")));

      stereo.setDisabledSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/RadioButtonDisabledSelected.png")));
      
      
      minBytes.setIcon(new ImageIcon
      (this.getClass().getResource("/files/images/RadioButtonNormalUnselected.png")));
            
      minBytes.setSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/RadioButtonNormalSelected.png")));

      minBytes.setDisabledIcon(new ImageIcon
      (this.getClass().getResource("/files/images/RadioButtonDisabledUnselected.png")));

      minBytes.setDisabledSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/RadioButtonDisabledSelected.png")));
         
         
      maxBytes.setIcon(new ImageIcon
      (this.getClass().getResource("/files/images/RadioButtonNormalUnselected.png")));
                     
      maxBytes.setSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/RadioButtonNormalSelected.png")));

      maxBytes.setDisabledIcon(new ImageIcon
      (this.getClass().getResource("/files/images/RadioButtonDisabledUnselected.png")));

      maxBytes.setDisabledSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/RadioButtonDisabledSelected.png")));
   
   
      transmit.setIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxNormalUnselected.png")));
                                                              
      transmit.setSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxNormalSelected.png")));
                                     
      transmit.setDisabledIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxDisabledUnselected.png")));
                                  
      transmit.setDisabledSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxDisabledSelected.png")));
      
      
      receive.setIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxNormalUnselected.png")));
                                                              
      receive.setSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxNormalSelected.png")));
                                     
      receive.setDisabledIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxDisabledUnselected.png")));
                                  
      receive.setDisabledSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxDisabledSelected.png")));
      
      
      record.setIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxNormalUnselected.png")));
                                                              
      record.setSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxNormalSelected.png")));
                                     
      record.setDisabledIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxDisabledUnselected.png")));
                                  
      record.setDisabledSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxDisabledSelected.png")));
   
   
      video.setIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxNormalUnselected.png")));
                                                               
      video.setSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxNormalSelected.png")));
                                      
      video.setDisabledIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxDisabledUnselected.png")));
                                  
      video.setDisabledSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxDisabledSelected.png")));
      
      
      play.setIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxNormalUnselected.png")));
                                                              
      play.setSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxNormalSelected.png")));
                                     
      play.setDisabledIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxDisabledUnselected.png")));
                                  
      play.setDisabledSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxDisabledSelected.png")));
      
      
      deleteAll.setIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxNormalUnselected.png")));
                                                              
      deleteAll.setSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxNormalSelected.png")));
                                     
      deleteAll.setDisabledIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxDisabledUnselected.png")));
                                  
      deleteAll.setDisabledSelectedIcon(new ImageIcon
      (this.getClass().getResource("/files/images/CheckBoxDisabledSelected.png")));
   }
   

   
   /**
    * Sets the grid bag layout.
    * ESCA-JAVA0076:
    */
   private void gridBagConstraints()
   {
      // Initialize grid bag layout of GUI.
      
      constraints = new GridBagConstraints();
      
      constraints.fill = GridBagConstraints.HORIZONTAL;
      
      
      
      // Grid bag constraints 1.
      
      constraints.insets = new Insets(10,0,0,0);
      
      
      constraints.ipadx = 0; 
      
      constraints.ipady = 0;

      constraints.gridwidth = 2;
      
      constraints.weightx = 0; 
      
      constraints.weighty = 1; 
      
      constraints.gridx = 0; 
      
      constraints.gridy = 0;
      
      
      contentPane.add(imagePane, constraints);
      
      
      // Grid bag constraints 2.
      
      constraints.insets = new Insets(10,1,0,1);
      
      
      constraints.ipadx = 0; 
      
      constraints.ipady = 0;

      constraints.gridwidth = 2;
      
      constraints.weightx = 0; 
      
      constraints.weighty = 0; 
      
      constraints.gridx = 0; 
      
      constraints.gridy = 1;
      
      
      contentPane.add(tabbedPane, constraints);
      
      
      // Grid bag constraints 3.
      
      constraints.insets = new Insets(20,1,10,1);
      
      
      constraints.ipadx = 0; 
      
      constraints.ipady = 0;

      constraints.gridwidth = 4;
      
      constraints.weightx = 0; 
      
      constraints.weighty = 0; 
      
      constraints.gridx = 0; 
      
      constraints.gridy = 2;
      
      
      contentPane.add(buttonPanel, constraints);
   }
   
   
   //*************************************** END INITIALISATION CODE ***************************************//
   
   
   //**************************************** START PUBLIC METHODS *****************************************//
   
   
   /**
    * Sets the visibility of the frame.
    */
   public void setVisibility()
   {
   
      if (isVisible) {frame.setVisible(true);}
      
      else {frame.setVisible(false);} 
   }
   
   
   
   /**
    * Clears the label icon.
    * @param icon
    */
   public static void clearLabelIcon(ImageIcon icon)
   {
      label.setIcon(icon); label.revalidate();
   }
   
   
   
   /**
    * Sets the label icon using a byte array.
    * @param videoArray
    * ESCA-JAVA0266:
    * ESCA-JAVA0166:
    */
   public static void setLabelIcon(byte[] videoArray)
   {
      try 
      {
         label.setIcon(new ImageIcon
         (
            ImageIO.read(new ByteArrayInputStream(videoArray)))
         );
      }
      
      catch (Exception e) 
      {
         System.out.println("Error refreshing the interface image frame.");
      }
   }
   
   
   
   /**
    * Appends information to the text output area.
    * @param value
    */
   public static void appendTextOutput(String value)
   {
      textOutput.append(value); textOutput.selectAll();
      
      int x = textOutput.getSelectionEnd();
      
      textOutput.select(x,x);
   }
   
   
   
   /**
    * Sets the messaging boolean variable.
    * @param value
    */
   public static void setIsMessaging(boolean value) {isMessaging = value;}
   
   
   
   /**
    * Returns the user id.
    * @return
    */
   public static String getUserId() {return userId;}
   
   
   
   /**
    * Returns the user id.
    * @return
    */
   public static String getUserPw() {return userPw;}
   
   
   //***************************************** END PUBLIC METHODS ******************************************//
   
   
   //**************************************** START PRIVATE METHODS ****************************************//
   
   
   /**
    * Sets the type of camera for the transceiver.
    */
   private void initialiseTransceiverWebcam()
   {
      if (camera.getSelectedItem().equals("1"))
      {
         avTransmit.setWebcamCamera(0);
      }
      
      if (camera.getSelectedItem().equals("2"))
      {
         avTransmit.setWebcamCamera(1);
      }
      
      if (camera.getSelectedItem().equals("3"))
      {
         avTransmit.setWebcamCamera(2);
      }
   }
   
   
   
   /**
    * Sets the type of camera for messaging.
    */
   private void initialiseMessagingWebcam()
   {
      if (camera.getSelectedItem().equals("1"))
      {
         avProcessor.setWebcamCamera(0);
      }
      
      if (camera.getSelectedItem().equals("2"))
      {
         avProcessor.setWebcamCamera(1);
      }
      
      if (camera.getSelectedItem().equals("3"))
      {
         avProcessor.setWebcamCamera(2);
      }
   }
   
   
   
   private class StartTransmitter extends Thread 
   {

      /** 
       * Thread that starts the transmitter.
       */
      private StartTransmitter()
      {
         setDaemon(false);
      }
      
      
      
      /**
       * ESCA-JAVA0166:
       * ESCA-JAVA0266:
       */
      public void run()
      {
         startTransmitter();  
      }
   }
   
   
   
   private class StartReceiver extends Thread 
   {

      /** 
       * Thread that starts the receiver.
       */
      private StartReceiver()
      {
         setDaemon(false);
      }
      
      
      
      /**
       * ESCA-JAVA0166:
       * ESCA-JAVA0266:
       */
      public void run()
      {
         startReceiver();
      }
   }
   
   
   
   private class StartMessaging extends Thread 
   {

      /** 
       * Thread that starts the receiver.
       */
      private StartMessaging()
      {
         setDaemon(false);
      }
      
      
      
      /**
       * ESCA-JAVA0166:
       * ESCA-JAVA0266:
       */
      public void run()
      {
         startMessaging();
      }
   }
   
   
   
   private class StopTransmitter extends Thread 
   {

      /** 
       * Thread that stops the transmitter.
       */
      private StopTransmitter()
      {
         setDaemon(false);
      }
      
      
      
      /**
       * ESCA-JAVA0166:
       * ESCA-JAVA0266:
       */
      public void run()
      {
         stopTransmitter();  
      }
   }
   
   
   
   private class StopReceiver extends Thread 
   {

      /** 
       * Thread that stops the receiver.
       */
      private StopReceiver()
      {
         setDaemon(false);
      }
      
      
      
      /**
       * ESCA-JAVA0166:
       * ESCA-JAVA0266:
       */
      public void run()
      {
         stopReceiver();  
      }
   }
   
   

   private class StopMessaging extends Thread 
   {

      /** 
       * Thread that starts the receiver.
       */
      private StopMessaging()
      {
         setDaemon(false);
      }
      
      
      
      /**
       * ESCA-JAVA0166:
       * ESCA-JAVA0266:
       */
      public void run()
      {
         stopMessaging();
      }
   }
   
   
   
   static class UppercaseDocumentFilter extends DocumentFilter 
   {
      
      /**
       * Tests the channel id insertion string for upper case.
       * @throws BadLocationException
       */
      public void insertString
      (
         DocumentFilter.FilterBypass fb, int offset, String text, AttributeSet attrs
      ) 
   
      throws BadLocationException {fb.insertString(offset, text.toUpperCase(), attrs);}

      
      /**
       * Converts the channel id insertion string to upper case.
       * @throws BadLocationException
       */
      public void replace
      (
         DocumentFilter.FilterBypass fb, int offset, int length, String text, AttributeSet attrs
      ) 
   
      throws BadLocationException {fb.replace(offset, length, text.toUpperCase(), attrs);}
   }
   
   
   
   /**
    * Starts the transmitter.
    * ESCA-JAVA0266:
    * ESCA-JAVA0166:
    */
   private void startTransmitter()
   {
      try
      {
         if (isPortValid())
         {
            // Create a audio transmit object with the specified parameters.
         
            String[] ports = dataTransmitPorts.getText().split(":");
            
            String transmitProtocol = (String)protocol.getSelectedItem();
            

            avTransmit = new AudioVideoTransmit
            (
               transmitProtocol, ipAddress.getText(), Integer.parseInt(ports[0]), Integer.parseInt(ports[1])
            );
            
            setTransmitterSettings();
      
         
            // Start the transmission.
      
            if (video.isSelected()) 
            {
               avTransmit.setIncludeVideo(true); initialiseTransceiverWebcam();
            }
         
         
            avTransmit.transmit(); appendTextOutput("\n Transmission has started.");
         
            transmit.setEnabled(false); receive.setEnabled(false);
         }

         else
         {
            appendTextOutput("\n Transmitter could not be started.");
         
            appendTextOutput("\n The port number may be invalid.");
            
            appendTextOutput("\n Port can't excede 5 digits.");
         }
      }
  
      catch (Exception e)
      {
         appendTextOutput("\n Transmitter could not be started.");
         
         appendTextOutput("\n The transmitter error is unknown.");
      }
   }
   
   
   
   /**
    * Stops the transmitter.
    * ESCA-JAVA0266:
    * ESCA-JAVA0166:
    */
   private void stopTransmitter()
   {
      try
      {  
         // Stop the transmission.
      
         avTransmit.setStopTransmit(true); avTransmit.cancel();
         
         
         appendTextOutput("\n Transmission has stopped."); 
         
         transmit.setEnabled(true); receive.setEnabled(true);
         
         
         setIsTransmitting(false); // Turns off the transmitter.
      }
 
      catch (Exception e)
      {
         appendTextOutput("\n Transmitter could not be stopped.");
         
         appendTextOutput("\n The transmitter error is unknown.");
      }
   }
   
   
   
   /**
    * Runs the receiver.
    * ESCA-JAVA0267:
    * ESCA-JAVA0166:
    * ESCA-JAVA0266:
    * ESCA-JAVA0076:
    * ESCA-JAVA0087:
    */
   private void startReceiver()
   {
      try
      {
         if (isPortValid() && isAddressValid())
         {
            // Create a audio receive object with the specified parameters.

            String[] ports = dataTransmitPorts.getText().split(":");
            
            String receiveProtocol = (String)protocol.getSelectedItem();
            
         
            avReceive = new AudioVideoReceive
            (
               receiveProtocol, ipAddress.getText(), Integer.parseInt(ports[0]), Integer.parseInt(ports[1])
            );

               
            // Start the reception.     
           
            avReceive.receive(); appendTextOutput("\n Reception has started.");
         
            transmit.setEnabled(false); receive.setEnabled(false);
         }
         
         else
         {
            appendTextOutput("\n Receiver could not be started.");
            
            appendTextOutput("\n The entered address may be invalid.");
         }
      }
        
      catch (Exception e)
      {
         appendTextOutput("\n Receiver could not be started.");
          
         appendTextOutput("\n The receiver error is unknown.");
      }
   }
   
   
   
   /**
    * Stops the receiver.
    * ESCA-JAVA0266:
    * ESCA-JAVA0166:
    */
   private void stopReceiver()
   {
      try
      {  
         // Stop the reception.
      
         avReceive.setStopReceive(true); avReceive.cancel();
      

         appendTextOutput("\n Reception has stopped.");
         
         transmit.setEnabled(true); receive.setEnabled(true);
         
         
         setIsReceiving(false); // Turns off the receiver.
      }
 
      catch (Exception e)
      {
         appendTextOutput("\n Receiver could not be stopped.");
         
         appendTextOutput("\n The receiver error is unknown.");
      }
   }
   
   
   
   /**
    * Starts the messaging.
    * ESCA-JAVA0266:
    * ESCA-JAVA0166:
    */
   private void startMessaging()
   {
      try
      {
         if (isVideoIdValid())
         {
            if (isWebServerReachable())
            {
               // Create a messaging object.

               avProcessor = new AudioVideoProcessor(videoId.getText());
         
         
               // Sets the mode of operation.
         
               if (play.isSelected()) 
               {
                  if (isChannelIdValid())
                  {
                     avProcessor.setModePlayback(true);
                     
                     avProcessor.setChannelId(channelId.getText());

                     
                     avProcessor.runProcessor(); // Start the messaging.
                  }
                  
                  else
                  {
                     setIsMessaging(false);
                  
                     appendTextOutput("\n Message could not be received.");
                      
                     appendTextOutput("\n A valid channel id is required.");
                  }
               }
         
               if (record.isSelected()) 
               {
                  avProcessor.setModeRecord(true);                
                  
                  setProcessorSettings(); initialiseMessagingWebcam();

                  
                  avProcessor.runProcessor(); // Start the messaging.
               }
            }
         }
         
         else
         {
            setIsMessaging(false); 
         
            appendTextOutput("\n Messaging could not be started.");
             
            appendTextOutput("\n The video id number may be invalid.");
              
            appendTextOutput("\n Video id can't excede 8 digits.");
            
            appendTextOutput("\n The video image size may be invalid.");
            
            appendTextOutput("\n Maximum video size is 350 x 240.");
         }
      }
  
      catch (Exception e)
      {
         setIsMessaging(false); 
      
         appendTextOutput("\n Messaging could not be started.");
         
         appendTextOutput("\n The messaging error is unknown.");
      }
   }
   
   
   
   /**
    * Stops the messaging.
    * ESCA-JAVA0266:
    * ESCA-JAVA0166:
    */
   private void stopMessaging()
   {
      try
      {
         if (getIsMessaging())
         {
            if (record.isSelected())
            {
               avProcessor.setStopRecord(true);

               appendTextOutput("\n Recording has stopped.");
            }
         
            if (play.isSelected())
            {
               avProcessor.setStopPlayback(true);
            
               appendTextOutput("\n Playback has stopped."); 
            }
         
            setIsMessaging(false);
         }
      }
 
      catch (Exception e)
      {
         appendTextOutput("\n Messaging could not be stopped.");
         
         appendTextOutput("\n The messaging error is unknown.");
      }
   }
   
   
   
   private class DeleteMessageListener implements ActionListener
   {
   
      /** 
       * Listener to delete recorded messages from database.
       * ESCA-JAVA0266 
       */
      public void actionPerformed(ActionEvent event)
      {  
         if ((userId != null) && (!runRestricted)
         && (!userId.equals("NON_MEMBER")))
         {
            clearDatabase();
         }
      }
   }
   
   
   
   private class SelectListener 
   implements ChangeListener
   {   
      
      /**
       * Listener constructor.
       * @param value
       */
      private SelectListener(int value)
      {
         checkBox = value;  
      }
   
      
      
      /**
       * Listener to enforce at least one
       * selection for transmit or receive.
       */
      public void stateChanged(ChangeEvent changeEvent) 
      {
         if (!isTransceiverChecked())
         {
            switch (checkBox)
            {
               case 0: 
        
                  transmit.setSelected(true); 
                  
               break;
        
                  
               case 1: 
               
                  receive.setSelected(true); 
                  
               break;
                  
                  
               default:
              
                  transmit.setSelected(true);
             
                  receive.setSelected(true); 
             
               break;
            }
         }
      }
      
      
      private int checkBox;
   };
   
   
   
   private class StartButtonListener
   implements ActionListener
   {
      /** 
       * Listener to start transmission or reception.
       * ESCA-JAVA0266 
       */
      public void actionPerformed(ActionEvent event)
      {
         if (transceiver.isSelected())
         {
            if (transmit.isSelected())
            {
               if (!getIsTransmitting())
               {
                  appendTextOutput("\n Starting transmitter."); 
               
                  Thread startTransmitter = new StartTransmitter();
               
                  startTransmitter.start(); setIsTransmitting(true);
               }
            }

         
            if (receive.isSelected())
            {
               if (!getIsReceiving())
               {
                  appendTextOutput("\n Starting receiver."); 
               
                  Thread startReceiver = new StartReceiver();
                
                  startReceiver.start(); setIsReceiving(true);
               }
            } 
         }
         
         
         if (messaging.isSelected())
         {
            if (!getIsMessaging())
            {
               appendTextOutput("\n Starting messaging.");
               
               Thread startMessaging = new StartMessaging();
             
               startMessaging.start(); setIsMessaging(true);
            }
         } 
         
          
         tabbedPane.setSelectedIndex(5);
      }
   }
   
   
   
   private class StopButtonListener
   implements ActionListener
   {
      /** 
       * Listener to stop transmission or reception.
       * ESCA-JAVA0266 
       */
      public void actionPerformed(ActionEvent event)
      {
         if (transceiver.isSelected())
         {
            if (transmit.isSelected())
            {
               if (getIsTransmitting())
               {
                  appendTextOutput("\n Stopping transmitter.");
               
                  Thread stopTransmitter = new StopTransmitter();
                
                  stopTransmitter.start();
               }
            }
         
            if (receive.isSelected())
            {
               if (getIsReceiving())
               {
                  appendTextOutput("\n Stopping receiver.");
               
                  Thread stopReceiver = new StopReceiver();
                
                  stopReceiver.start();
               }
            }          
         }
         
         
         if (messaging.isSelected())
         {
            if (getIsMessaging())
            {
               appendTextOutput("\n Stopping messaging.");             
               
               Thread stopMessaging = new StopMessaging();

               stopMessaging.start();
            }
         }
      }
   }
   
   

   /**
    * Adds a settings menu. 
    * @return menu
    */
   private JMenu createSettingsMenu()
   {
      JMenu menu = new JMenu("Mode"); 
      

      transceiver = new JRadioButtonMenuItem("Transceiver");
      
      messaging = new JRadioButtonMenuItem("Messaging");
      

      ButtonGroup group = new ButtonGroup();
      
      
      group.add(transceiver); group.add(messaging);
      
      group.add(delete); menu.add(transceiver); 
      
      menu.add(messaging);
      
      
      if ((userId == null) || (runRestricted)
      || (userId.equals("NON_MEMBER")))
      {
         messaging.setEnabled(false);
      }
      
      
      transceiver.setSelected(true);

      
      return menu;
   }
   
   
   
   /**
    * Tests if an address is reachable.
    * ESCA-JAVA0266:
    * @param address
    * @return
    */
   @SuppressWarnings("unused")
   private static boolean isAddressReachable(String address)
   {
      boolean reachable = false;
      
      
      try
      {
         if (InetAddress.getByName(address).isReachable(PING_TIMEOUT))
         {
            reachable = true;
         }
      }
      
      
      catch (IOException e)
      {
         appendTextOutput("\n An unknown connection error occured.");
      }
      

      return reachable;   
   }
   
   
   
   /**
    * Tests if the web server is reachable.
    * @return
    */
   private static boolean isWebServerReachable()
   {
      boolean valid = false; 

      
      if (isWebServerReachable(DataUploadDownload.getFileDownloadAddress())
      && isWebServerReachable(DataUploadDownload.getFileUploadAddress()))
      {
         valid = true;
      }
      
      
      return valid;  
   }
   
   
   
   /**
    * Validation test for channel id.
    * @return
    */
   private boolean isChannelIdValid()
   {
      boolean valid = false; 

      
      if (validation.alphanumericValidation(channelId.getText()) 
      && (channelId.getText().length() <= MAX_CHANNEL_ID_LENGTH)
      && (channelId.getText().length() >= MIN_CHANNEL_ID_LENGTH))
      {
         valid = true;
      }
      
      
      return valid;
   }
   
   
   
   /**
    * Validation test for video id.
    * @return
    */
   private boolean isVideoIdValid()
   {
      boolean valid = false; 

      
      if (validation.integerValidation(videoId.getText()) 
      && (videoId.getText().length() <= MAX_VIDEO_ID_LENGTH))
      {
         valid = true;
      }
      
      
      return valid;
   }
   
   
   
   /**
    * Validation test for port.
    * @return
    */
   private boolean isPortValid()
   {
      boolean valid = false; 

      
      String[] ports = dataTransmitPorts.getText().split(":");
      
      if ((validation.integerValidation(ports[0])) && (ports[0].length() <= MAX_PORT_LENGTH)
      && (validation.integerValidation(ports[1])) && (ports[1].length() <= MAX_PORT_LENGTH))
      {
         valid = true;
      }
      
      
      return valid;
   }
   
   
   
   /**
    * Validation test for address.
    * @return
    */
   private boolean isAddressValid()
   {
      boolean valid = false; 

      
      if (validation.ipAddressValidation(ipAddress.getText()))
      {
         valid = true;
      }
      
      
      return valid;
   }
   
   
   
   /**
    * Tests if the web server is reachable.
    * ESCA-JAVA0266:
    * @param address
    * @return
    */
   private static boolean isWebServerReachable(String address)
   {
      final int SUCCESS_CODE = 200; boolean reachable = false;
      
      
      try
      {  
         URL url = new URL(address);
    
         HttpURLConnection huc = (HttpURLConnection)url.openConnection(); 
    
         huc.setRequestMethod("GET");  // or huc.setRequestMethod("HEAD"); 
    
         huc.connect(); int code = huc.getResponseCode();
         
         
         if (code == SUCCESS_CODE) {reachable = true;}
      }
      
     
      catch (IOException e)
      {
         setIsMessaging(false); // Turns off the messaging. 
      
         appendTextOutput("\n Messaging could not be started.");
          
         appendTextOutput("\n Could not connect to the web server.");
      }
      

      return reachable;   
   }
   
   
   
   /**
    * Updates the progress bar.
    * @param text
    * @param indeterminate
    */
   public static void updateProgressBar(String text, boolean indeterminate)
   {
      progressBar.setIndeterminate(indeterminate);
   
      progressBar.setString(text);   
   }
   
   
   
   /**
    * Sets the settings for the processor.
    */
   private void setProcessorSettings()
   {
      final int videoQualityDivider = 100; final float maxVideoQualityFactor = 0.75f;
      
      final int frameRateDivider = 3;
 
      
      avProcessor.setAudioSamples(Integer.parseInt((String)samples.getSelectedItem()));
      
      avProcessor.setFrameRate((chunks.getValue()+frameRateDivider)/frameRateDivider);
      
      
      avProcessor.setVideoQuality
      (
         ((float)(quality.getValue())/videoQualityDivider) * maxVideoQualityFactor
      );
      
      
      videoSizeInfo = ((String)size.getSelectedItem()).split("x");
      
      
      avProcessor.setVideoHeight(Integer.parseInt(videoSizeInfo[1]));
      
      avProcessor.setVideoWidth(Integer.parseInt(videoSizeInfo[0]));
       
       
      if (mono.isSelected()) {avProcessor.setAudioChannels(1);}
       
      if (stereo.isSelected()) {avProcessor.setAudioChannels(2);}
       
       
      if (minBytes.isSelected()) {avProcessor.setAudioBytes(1);}
       
      if (maxBytes.isSelected()) {avProcessor.setAudioBytes(2);}
      
      
      if (video.isSelected()) {avProcessor.setIncludeVideo(true);}
      
      else {avProcessor.setIncludeVideo(false);}
   }
   

   
   /**
    * Sets the settings for the transmitter.
    */
   private void setTransmitterSettings()
   {
      final int videoQualityDivider = 100; final float maxVideoQualityFactor = 0.75f;
      
      final int minFrameRate = 25; final double rateMultiplyFactor = 4.75;
      
      final int frameRateInvert = 100;
      

      avTransmit.setAudioSamples(Float.parseFloat((String)samples.getSelectedItem()));
      
      avTransmit.setAudioChunkFactor(chunks.getValue());
      
      
      avTransmit.setFrameRate
      (
         minFrameRate + (int)((frameRateInvert - rate.getValue()) * rateMultiplyFactor)
      );
      
      
      avTransmit.setVideoQuality
      (
         ((float)(quality.getValue())/videoQualityDivider) * maxVideoQualityFactor
      );
     
      
      videoSizeInfo = ((String)size.getSelectedItem()).split("x");
      
      
      avTransmit.setVideoHeight(Integer.parseInt(videoSizeInfo[1]));
      
      avTransmit.setVideoWidth(Integer.parseInt(videoSizeInfo[0]));
       
       
      if (mono.isSelected()) {avTransmit.setAudioChannels(1);}
       
      if (stereo.isSelected()) {avTransmit.setAudioChannels(2);}
       
       
      if (minBytes.isSelected()) {avTransmit.setAudioBytes(1);}
       
      if (maxBytes.isSelected()) {avTransmit.setAudioBytes(2);}
      
      
      if (video.isSelected()) {avTransmit.setIncludeVideo(true);}
      
      else {avTransmit.setIncludeVideo(false);}
   }
   
   
   
   /**
    * Clears the messages from the database.
    */
   private void clearDatabase()
   {
      ArrayList<String> data = new ArrayList<String>();   

      
      if (deleteAll.isSelected())
      {
         Portal.clearFromDatabase(MESSAGE_CLEAR_ALL, userId, userPw, null); 
      }
       
      else
      {
         data.add(videoId.getText());
      
         Portal.clearFromDatabase(MESSAGE_CLEAR_ID, userId, userPw, data); 
      }   
   }

   
   
   /**
    * Window resize event.
    */
   private void includeResizeEvent()
   {
      this.addComponentListener 
      (
         new ComponentAdapter() 
         {
            public void componentResized(ComponentEvent e) 
            {
               appletDisplayHandler();
            }
          
         }
      ); 
   }
   
   
   
   /**
    * Controls the JApplet component visibility. 
    * ESCA-JAVA0266:
    */
   private void appletDisplayHandler()
   {
      final String ZOOM_WARNING = 
   
      "It appears that web pages in your browser are zoomed out. \n\n" +
      "In order to display the applet, your browser needs to be set \n" +
      "at its default zoom level of 100% or greater. \n\n" +
      "Sorry about that.";
   
   
      if ((getWidth() < FRAME_SIZE_X) || (getHeight() < APPLET_SIZE_Y))
      {
         contentPane.setVisible(false); menuBar.setVisible(false);
      
         
         if (!zoomWarningShown) 
         {
            Message.showMessage(ZOOM_WARNING, "Browser zoom setting", "warning");
            
            zoomWarningShown = true;
         } 
      }
       
      else 
      {
         contentPane.setVisible(true); menuBar.setVisible(true);
      } 
   }
   
   
   
   /**
    * Tests if at least one transceiver check box is selected.
    * @return
    */
   private boolean isTransceiverChecked()
   {
      if (transmit.isSelected()) {return true;}
      
      else if (receive.isSelected()) {return true;}
      
      else {return false;}
   }
   
   
   
   /**
    * Sets the transmission boolean variable.
    * @param value
    */
   private void setIsTransmitting(boolean value) {isTransmitting = value;}
   
   
   
   /**
    * Sets the reception boolean variable.
    * @param value
    */
   private void setIsReceiving(boolean value) {isReceiving = value;}
   
   
   
   /**
    * Gets the recording boolean variable.
    * @return
    */
   private static boolean getIsMessaging() {return isMessaging;}
   
   

   /**
    * Gets the transmission boolean variable.
    * @return
    */
   private boolean getIsTransmitting() {return isTransmitting;}
   
   
   
   /**
    * Gets the reception boolean variable.
    * @return
    */
   private boolean getIsReceiving() {return isReceiving;}
   

   //***************************************** END PRIVATE METHODS *****************************************//
   
   
   //********************************************** START MAIN *********************************************//
   
   
   /**
    * Main
    * @param args
    */
   public static void main(String[] args)
   {
      isApplication = true;
      
      isVisible = true;
   
      new OnlineMessagingGui();
   }
   
   
   //*********************************************** END MAIN **********************************************//
   
  
   /**
    * ESCA-JAVA0007:
    */
   public JFrame frame;

   private JMenuBar menuBar;
   private static JLabel label;
   private Validation validation;
   private Container contentPane;
   private DocumentFilter filter;
   private String[] videoSizeInfo;
   private JTabbedPane tabbedPane;
   private static boolean isMessaging;
   private static JTextArea textOutput;
   private AudioVideoReceive avReceive;
   private JButton delete, start, stop;
   private static JScrollPane textPane;
   private static JScrollPane imagePane;
   private static String userId, userPw;
   private JSlider quality, rate, chunks;
   private AudioVideoTransmit avTransmit;
   private GridBagConstraints constraints;
   private AudioVideoProcessor avProcessor;
   private static JProgressBar progressBar;
   private ButtonGroup channelsGroup, bytesGroup;
   private JRadioButtonMenuItem transceiver, messaging;
   private JRadioButton mono, stereo, minBytes, maxBytes;
   private JComboBox<String> protocol, camera, samples, size;
   private boolean isTransmitting, isReceiving, zoomWarningShown;
   private JCheckBox deleteAll, transmit, receive, video, record, play;
   private JTextField channelId, videoId, deleteId, ipAddress, dataTransmitPorts;

   private String[] cameraChoices = {"1", "2", "3"};
   private String[] protocolChoices = {"TCP", "UDP"};
   private String[] sizeChoices = {"176x144", "320x240", "432x240"};
   private String[] samplesChoices = {"8000", "16000", "32000", "44100", "48000"};

   private JPanel imagePanel, buttonPanel, transceiverPanel, frameRatePanel,
   recordPanel, protocolPanel, playPanel, chunkSizePanel, deletePanel, videoQualityPanel,
   videoSizePanel, messagingPanel, videoPanel, audioPanel, channelsPanel, feedbackPanel, bytesPanel,
   transmitPanel, receivePanel, videoIdPanel, videoIncPanel, cameraPanel, samplesPanel, deleteAllPanel;
}


//********************************************** END CLASS **********************************************//