/*
   Copyright (c) 2010 - 2013 Javaika.com by Alexander Wait.
   
   All Rights Reserved.
*/



package required;

import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;

import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageWriter;
import javax.imageio.stream.ImageOutputStream;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.TargetDataLine;
import javax.swing.ImageIcon;
import javax.swing.JFrame;

import com.github.sarxos.webcam.Webcam;

import main.OnlineMessagingGui;


//********************************************* START CLASS *********************************************//


/**
 * This class records and plays video and audio.
 * 
 * @author Alexander Wait
 * @version 2014-07-25
 */
public class AudioVideoProcessor extends JFrame
{
   private static final long serialVersionUID = 1L;
   
   
   private static final int SQL_TABLE_COLUMN_A = 0;
   
   private static final int SQL_TABLE_COLUMN_B = 1;
   
   private static final int SQL_TABLE_COLUMN_C = 2;
   
   private static final int SQL_TABLE_COLUMN_D = 3;
   
   private static final int SQL_TABLE_COLUMN_E = 4;


   /* Configuration Settings */
   
   private static final int MAX_RECORD_BYTE_SIZE = 30000000; // 30 Megabyte maximum.

   private static final String FLAG = "*-^-*"; // Key to separate multiplex frames.
   
   private static boolean VIDEO = false; // Video recording is disabled by default.
   
   
   private static final String MESSAGE_WRITE_ADDRESS = "http://" + Domain.getDomain() 
   
   + "/bridge/record-message-config-write.php" + Domain.getPort(); // Port:80
   
   
   private static final String MESSAGE_READ_ADDRESS = "http://" + Domain.getDomain() 
   
   + "/bridge/record-message-config-read.php" + Domain.getPort(); // Port:80
   

   //************************************** START INITIALISATION CODE **************************************//


   /**
    * Class constructor.
    * @param videoId
    * ESCA-JAVA0266:
    */
   public AudioVideoProcessor(String videoId)
   {
      videoThumbnail = new ImageIcon(this.getClass().getResource
      
      ("files/images/VideoMessagingImageBlank.png"));
   
   
      dataUploadDownload = new DataUploadDownload();

      videoStream = new ByteArrayOutputStream();
      
      setMessageId(videoId);
   }
   
   
   //*************************************** END INITIALISATION CODE ***************************************//
   
   
   //**************************************** START PUBLIC METHODS *****************************************//
   
   
   /**
    * Runs the message processor.
    * ESCA-JAVA0166:
    * ESCA-JAVA0266:
    */
   public void runProcessor()
   {
      try
      {
         if (getModeRecord())
         {
            setBufferArray(); record(); 
            
            transferToDatabase(); encodeBytes(); exportBytes(); 
         }   
      
         if (getModePlayback())
         {
            importBytes(); readFromDatabase();
            
            setBufferArray(); decodeBytes(); playback();
         }
      }
      
      
      catch (Exception e) 
      { 
         System.out.println("Error running the processor.");
      }
   }
   
   
   
   /**
    * Stops the recording.
    * ESCA-JAVA0266:
    * ESCA-JAVA0166:
    */
   public void cancel()
   { 
      stopRefresh = true;  
   
   
      try
      { 
         if (targetDataLine != null)  
         { 
            if (targetDataLine.isOpen()) {targetDataLine.close();}
         }
         
         if (sourceDataLine != null)  
         { 
            if (sourceDataLine.isOpen()) {sourceDataLine.close();}
         }

         if (videoStream != null) {videoStream.close();}
         
         if (webcam != null) {webcam.close();}
      }
      
      catch (IOException e) {System.out.println(e);} 
   }
   
   
   
   /**
    * Sets the video inclusion boolean variable.
    * @param value
    */
   public void setIncludeVideo(boolean value) {VIDEO = value;}
   
   
   
   /**
    * Sets the current record size.
    * @param val
    */
   public void setCurrentSize(int val) {currentSize = val;}
   
   
   
   /**
    * Sets the audio samples.
    * @param val
    */
   public void setAudioSamples(int val) {audioSamples = val;}
   
   
   
   /**
    * Sets the video quality.
    * @param val
    */
   public void setVideoQuality(float val) {videoQuality = val;}
   
   
   
   /**
    * Sets the channel id.
    * @param val
    */
   public void setChannelId(String val) {channelId = val;}
   
   
   
   /**
    * Sets the message id.
    * @param val
    */
   public void setMessageId(String val) {messageId = val;}
   
   
   
   /**
    * Sets the audio channels.
    * @param val
    */
   public void setAudioChannels(int val) {audioChannels = val;}
   
   
   
   /**
    * Sets the frame rate.
    * @param val
    */
   public void setFrameRate(int val) {frameRate = val;}
   
   
   
   /**
    * Sets the audio byte number.
    * @param val
    */
   public void setAudioBytes(int val) {audioBytes = val;}
   
   
   
   /**
    * Sets the video width.
    * @param val
    */
   public void setVideoWidth(int val) {videoWidth = val;}
   
   
   
   /**
    * Sets the video height.
    * @param val
    */
   public void setVideoHeight(int val) {videoHeight = val;}
   
   
   
   /**
    * Sets the record boolean.
    * @param val
    */
   public void setModeRecord(boolean val) {modeRecord = val;}
   
   
   
   /**
    * Sets the play boolean.
    * @param val
    */
   public void setModePlayback(boolean val) {modePlayback = val;}
   
   
   
   /**
    * Sets the record boolean.
    * @param val
    */
   public void setStopRecord(boolean val) {stopRecord = val;}
   
   
   
   /**
    * Sets the play boolean.
    * @param val
    */
   public void setStopPlayback(boolean val) {stopPlayback = val;}
   
   
   
   /**
    * Sets the camera variable.
    * @param value
    */
   public void setWebcamCamera(int value) {webcamCamera = value;}
   
   
   //***************************************** END PUBLIC METHODS ******************************************//
   
   
   //**************************************** START PRIVATE METHODS ****************************************//  

   
   private class StartRefreshing extends Thread 
   {

      /** 
       * Thread that starts frame refreshing.
       */
      private StartRefreshing() 
      {
         setDaemon(false);
      }
      
      
      
      /**
       * ESCA-JAVA0166:
       * ESCA-JAVA0266:
       */
      public void run()
      {
         refreshImageFrame();  
      }
   }
   
   
   
   /**
    * Sets the label icon using a byte array.
    * @param videoArray
    * ESCA-JAVA0266:
    */
   private static void setLabelIcon(byte[] videoArray)
   {
      OnlineMessagingGui.setLabelIcon(videoArray);
   }
   
   
   
   /**
    * Starts the web camera.
    * ESCA-JAVA0266:
    */
   private void startWebcam()
   {
      webcam = Webcam.getWebcams().get(getWebcamCamera());
      
      webcam.setCustomViewSizes(new Dimension[] {new Dimension(getVideoWidth(), getVideoHeight())});

      webcam.setViewSize(new Dimension(getVideoWidth(), getVideoHeight())); webcam.open();
   }
   
   
   
   /**
    * Sets the audio buffer array.
    */
   private void setBufferArray()
   {
      int testSize = (int)Math.floor
      (
         (double)(getAudioSamples() * getAudioBytes() * getAudioChannels()) / getFrameRate()
      ); 
      

      int frameSize = getAudioBytes() * getAudioChannels();
      
     
      for (int i = 0; i < frameSize; i ++)
      {
         if ((testSize % frameSize) == 0)
         {
            audioBufferSize = testSize; break;
         }
         
         else {testSize --;}
      }
      

      audioByteArray = new byte[getAudioBufferSize()];
   }
   
   
   
   /**
    * Starts the target data line.
    * ESCA-JAVA0266:
    */
   private void startTargetLine()
   {   
      DataLine.Info dataLineInfo = new DataLine.Info(TargetDataLine.class, getAudioFormat());
      
        
      try
      {
         targetDataLine = (TargetDataLine) AudioSystem.getLine(dataLineInfo);
         
         targetDataLine.open(getAudioFormat());
      }
         
      catch (LineUnavailableException e)
      { 
         System.out.println("The audio system line is unavailable.");
      }
         
         
      targetDataLine.start();         
   }
   
   
   
   /**
    * Starts the source data line.
    * ESCA-JAVA0266:
    */
   private void startSourceLine()
   {
      DataLine.Info dataLineInfo = new DataLine.Info(SourceDataLine.class, getAudioFormat());
      
      
      try
      {      
         sourceDataLine = (SourceDataLine) AudioSystem.getLine(dataLineInfo);
      
         sourceDataLine.open(getAudioFormat());        
      }
      
      catch (LineUnavailableException e)
      {
         System.out.println("The audio system line is unavailable.");
      }
      
      
      sourceDataLine.start();
   }
   
   

   /**
    * Returns the audio format.
    * @return
    */
   private AudioFormat getAudioFormat()
   {
      boolean signed = true; boolean bigEndian = false;
    
      
      return new AudioFormat
      (
         getAudioSamples(), (8 * getAudioBytes()), getAudioChannels(), signed, bigEndian
      );
   }
   
   

   /**
    * Refreshes the camera frame.
    */
   private void refreshImageFrame()
   {
      while (!stopRefresh)
      {  
         image = webcam.getImage();
      }
   }
   
   
   
   /**
    * Reads settings from the database.
    */
   private void readFromDatabase()
   {
      readSettings = new ArrayList<String>(); writeSettings = new ArrayList<String>();
      
      writeSettings.add(getMessageId()); writeSettings.add(getChannelId());
      
      
      readSettings = Portal.phpToJavaGate
      (
         MESSAGE_READ_ADDRESS, OnlineMessagingGui.getUserId(), 
         
         OnlineMessagingGui.getUserPw(), writeSettings
      );
      
      
      if (readSettings.size() != 0)
      {
         setAudioChannels(Integer.parseInt(readSettings.get(SQL_TABLE_COLUMN_A)));
         
         setAudioSamples(Integer.parseInt(readSettings.get(SQL_TABLE_COLUMN_B)));
         
         setAudioBytes(Integer.parseInt(readSettings.get(SQL_TABLE_COLUMN_C)));
         
         setFrameRate(Integer.parseInt(readSettings.get(SQL_TABLE_COLUMN_D)));
         
         
         if (readSettings.get(SQL_TABLE_COLUMN_E).equals("1")) // 1 = on, 0 = off
         {
            setIncludeVideo(true);
         }
      }
   }
   

   
   /**
    * Starts the recording.
    * ESCA-JAVA0265:
    * ESCA-JAVA0087:
    */
   private void record()
   {
      OnlineMessagingGui.appendTextOutput("\n Getting ready to record.");
      
      
      startTargetLine(); // Start the audio data line for audio/video recording.
      
      
      if (isIncludeVideo())
      {  
         startWebcam(); frameRefreshing = new StartRefreshing(); frameRefreshing.start();
      }
      
      
      try {Thread.sleep(5000);} catch (InterruptedException e1) {e1.printStackTrace();}
      
      
      OnlineMessagingGui.appendTextOutput("\n Recording has started.");
      
      messageBytes.add(getFlagBytes()); // Adds flag bytes before record start.

     
      while (!(getStopRecord()) && !(getCurrentSize() > MAX_RECORD_BYTE_SIZE))
      {
         OnlineMessagingGui.updateProgressBar
         (
            "Recording Message (" + returnProgressPercentage() + "%)", false
         );
      

         if (isIncludeVideo())
         {
            recordAudioVideo(); // Record audio and video bytes if video enabled.
         }
         
         
         else
         {
            recordAudio(); // Record only audio bytes if video is disabled.
         }
         
         
         messageBytes.add(getFlagBytes()); // Adds the break flag.
         
         
         setCurrentSize(getMessageSize()); // Sets current size.
      }
      

      OnlineMessagingGui.appendTextOutput("\n Recording has completed.");
      
      OnlineMessagingGui.updateProgressBar("Recording Complete", false);

      
      disableCameraRefresh(); cancel();
   }
   
   
   
   /**
    * Record both audio and video.
    */
   private void recordAudioVideo()
   {
      compressImage(image); audioByteArray = new byte[getAudioByteArrayLength()]; 
       

      targetDataLine.read(getAudioByteArray(), 0, getAudioByteArrayLength()); 
      
      
      byte[] audioData = getAudioByteArray(); byte[] videoData = getVideoByteArray();
      
      
      byte[] combine = new byte[audioData.length + videoData.length];
       
       
      System.arraycopy(audioData, 0, combine, 0, audioData.length);
       
      System.arraycopy(videoData, 0, combine, audioData.length, videoData.length);
      
      
      messageBytes.add(combine); resetVideoStream();   
   }
   
   
   
   /**
    * Play both audio and video.
    * @param index
    */
   private void playAudioVideo(int index)
   {
      byte[] audioData = Arrays.copyOfRange
      (
         messageBytes.get(index), 0, getAudioBufferSize()
      );

       
      byte[] videoData = Arrays.copyOfRange
      (
         messageBytes.get(index), getAudioBufferSize(), messageBytes.get(index).length
      );   


      sourceDataLine.write(audioData, 0, audioData.length); setLabelIcon(videoData);
   }
   
   
   
   /**
    * Record only the audio.
    */
   private void recordAudio()
   {
      audioByteArray = new byte[getAudioByteArrayLength()]; 
       

      targetDataLine.read(getAudioByteArray(), 0, getAudioByteArrayLength());  
   
      
      messageBytes.add(getAudioByteArray());   
   }
   
   
   
   /**
    * Play only the audio.
    * @param index
    */
   private void playAudio(int index)
   {
      sourceDataLine.write(messageBytes.get(index), 0, messageBytes.get(index).length);
   }
   
   
   
   /**
    * Returns the progress percentage.
    * @return
    */
   private int returnProgressPercentage()
   {
      double result = ((double)getCurrentSize() / MAX_RECORD_BYTE_SIZE) * 100;
      
      return (int)result;
   }
   
   
   
   /**
    * Compresses the buffered image before storage.
    * ESCA-JAVA0266:
    * @param img
    */
   private void compressImage(BufferedImage img)
   {     
      try
      {          
         ByteArrayOutputStream baos = new ByteArrayOutputStream();
         
         ImageOutputStream ios = ImageIO.createImageOutputStream(baos);
        
         
         Iterator<ImageWriter> writers = ImageIO.getImageWritersByFormatName("jpg");
         
         ImageWriter writer = writers.next();
          
         
         ImageWriteParam param = writer.getDefaultWriteParam();
         
         
         param.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
         
         param.setCompressionQuality(getVideoQuality()); writer.setOutput(ios); 
         
         writer.write(null, new IIOImage(img, null, null), param);
             
         
         byte[] dump = baos.toByteArray(); writer.dispose(); 
             
         getVideoStream().write(dump, 0, dump.length);         
      }
        
      catch (IOException e)
      { 
         System.out.println("An error has occured during image compression.");
      }
   }
   
   

   /**
    * Starts the play back.
    */
   private void playback()
   {
      startSourceLine(); OnlineMessagingGui.appendTextOutput("\n Playback has started.");
      
      
      OnlineMessagingGui.updateProgressBar("Playing Message", false); final int inc = 2;
      
      
      for (int i = 1; i < (indexes.size() / inc) * inc; i += inc)  
      {
         if (isIncludeVideo())
         {
            playAudioVideo(i); // Play audio and video bytes if video enabled. 
         }

         else
         {    
            playAudio(i); // Play only audio bytes if video is disabled.
         }
         
         
         if (getStopPlayback()) {break;} 
      }
      
      if (!getStopPlayback()) 
      {
         OnlineMessagingGui.appendTextOutput("\n Playback has completed.");
         
         OnlineMessagingGui.updateProgressBar("Playback Complete", false);
         
         OnlineMessagingGui.setIsMessaging(false);
      }
      
      
      OnlineMessagingGui.clearLabelIcon(videoThumbnail); cancel();
   }
   
   
   
   /**
    * Encodes bytes to an array.
    * ESCA-JAVA0266:
    */
   private void encodeBytes()
   {
      data = new byte[getMessageSize()]; 
      
      
      int index = 0; int size = messageBytes.size();
  
  
      for (int i = 0; i < size; i++)
      {
         byte[] dump = messageBytes.get(i);
         
            
         for (int j = 0; j < dump.length; j ++)
         {
            data[j + index] = dump[j];
         }
         
         
         index += dump.length; 
      }
   }
   
   
   
   /**
    * Decodes bytes to array lists.
    */
   private void decodeBytes()
   {
      addIndexes(); messageBytes.add(getFlagBytes()); final int inc = 2;


      for (int i = 0; i < (indexes.size() / inc) * inc; i += inc)
      {  
         messageBytes.add
         (
            Arrays.copyOfRange(data, indexes.get(i), indexes.get(i + 1) +1)
         );
         

         messageBytes.add(getFlagBytes());
      }
   }
   


   /**
    * Gets the size of message bytes.
    * @return
    */
   private int getMessageSize()
   {
      int size = 0;

      for (int i = 0; i < messageBytes.size(); i++)
      {
         size += messageBytes.get(i).length;
      }
      

      return size;
   }
   
   
   
   /**
    * Imports the bytes.
    */
   private void importBytes()
   {
      data = dataUploadDownload.importMessageData
      (
         OnlineMessagingGui.getUserId(),
         
         OnlineMessagingGui.getUserPw(),
         
         getMessageId(), getChannelId()
      );
   }
   
   
   
   /**
    * Exports the bytes.
    */
   private void exportBytes()
   {
      dataUploadDownload.exportMessageData
      (
         OnlineMessagingGui.getUserId(),
             
         OnlineMessagingGui.getUserPw(),
         
         getMessageId(), data
      );
   }
   
   
   
   /**
    * Gets the flag byte array.
    * @return
    */
   private static byte[] getFlagBytes()
   {
      byte[] flagArray = new byte[FLAG.length()];
   
      
      for (int i = 0; i < flagArray.length; i ++)
      {
         flagArray[i] = (byte) FLAG.charAt(i);
      }
      
      
      return flagArray;
   }
   
   
   
   /**'
    * Tests if an index position is the start of a frame.
    * @param index
    * @return
    */
   private boolean isFrameStart(int index)
   {
      int count = 0;
      
      
      if (index > (FLAG.length() -1))
      {   
         for (int i = 1; i <= FLAG.length(); i ++)
         {
            if ((char)data[index - i] == FLAG.charAt(FLAG.length() - i))
            {
               count ++;
            }
         }
      
         if (count == FLAG.length()) {return true;}
      
         else {return false;}
      }
      
      else {return false;}
   }
   
   
   
   /**'
    * Tests if an index position is the end of a frame.
    * @param index
    * @return
    */
   private boolean isFrameStop(int index)
   {
      int count = 0;
      
      
      if (index < (data.length - FLAG.length()))
      {   
         for (int i = 1; i <= FLAG.length(); i ++)
         {
            if ((char)data[index + i] == FLAG.charAt(i - 1))
            {
               count ++;
            }
         }
      
         if (count == FLAG.length()) {return true;}
      
         else {return false;}
      }
      
      else {return false;}
   }
   
   

   /**
    * Adds the frame start-stop indexes to array lists.
    */
   private void addIndexes()
   {
      for (int i = 0; i < data.length; i ++)
      {
         if (isFrameStart(i) || isFrameStop(i)) {indexes.add(i);}
      }
   }
   
   
   
   /**
    * Transfers the settings to the database.
    */
   private void transferToDatabase()
   {
      writeSettings = new ArrayList<String>();

      
      writeSettings.add(getMessageId());
      
      writeSettings.add(getIncludeVideo());
      
      writeSettings.add(Integer.toString(getAudioChannels()));
      
      writeSettings.add(Integer.toString(getAudioSamples()));
      
      writeSettings.add(Integer.toString(getAudioBytes()));
      
      writeSettings.add(Integer.toString(getFrameRate()));
      
      
      Portal.javaToPhpGate
      (
         MESSAGE_WRITE_ADDRESS, OnlineMessagingGui.getUserId(), 
         
         OnlineMessagingGui.getUserPw(), writeSettings
      );  
   }
   

   
   /**
    * Disables the camera refreshing.
    */
   private void disableCameraRefresh() {stopRefresh = true;}
   
   
   
   /**
    * Resets the byte array output stream.
    */
   private void resetVideoStream() {videoStream.reset();}
   
   
   
   /**
    * Gets the video inclusion boolean variable.
    * @return
    */
   private static boolean isIncludeVideo() {return (VIDEO);}
   
   
   
   /**
    * Returns 1 if video is included, otherwise 0.
    * @return
    */
   private static String getIncludeVideo() 
   {
      if (isIncludeVideo()) {return "1";} else {return "0";}
   }
   
   
   
   /**
    * Gets the current record size.
    * @return
    */
   private int getCurrentSize() {return currentSize;}
   

   
   /**
    * Gets the audio samples.
    * @return samples
    */
   private int getAudioSamples() {return audioSamples;}
   
   
   
   /**
    * Gets the audio quality.
    * @return quality
    */
   private float getVideoQuality() {return videoQuality;}
   
   
   
   /**
    * Gets the channel id.
    * @return channels
    */
   private String getChannelId() {return channelId;}
   
   
   
   /**
    * Gets the message id.
    * @return channels
    */
   private String getMessageId() {return messageId;}
   
   
   
   /**
    * Gets the audio channels.
    * @return channels
    */
   private int getAudioChannels() {return audioChannels;}
   
   
  
   /**
    * Gets the frame rate.
    * @return frames
    */
   private int getFrameRate() {return frameRate;}
   
   
   
   /**
    * Gets the audio byte number.
    * @return bytes
    */
   private int getAudioBytes() {return audioBytes;}
   
   
   
   /**
    * Gets the video width.
    * @return width
    */
   private int getVideoWidth() {return videoWidth;}
   

   
   /**
    * Gets the video height.
    * @return width
    */
   private int getVideoHeight() {return videoHeight;}
   
    
   
   /**
    * Gets the record boolean.
    * @return
    */
   private boolean getModeRecord() {return modeRecord;}
   
   
   
   /**
    * Gets the play boolean.
    * @return
    */
   private boolean getModePlayback() {return modePlayback;}
   
   
   
   /**
    * Gets the record boolean.
    * @return
    */
   private boolean getStopRecord() {return stopRecord;}
   
   
   
   /**
    * Gets the play boolean.
    * @return
    */
   private boolean getStopPlayback() {return stopPlayback;}
   
   
   
   /**
    * Gets the camera to use for the message.
    * @return
    */
   private int getWebcamCamera() {return (webcamCamera);}
   
   
   
   /**
    * Gets the video output stream.
    * @return
    */
   private ByteArrayOutputStream getVideoStream() {return (videoStream);}
   
   
   
   /**
    * Gets the audio buffer size.
    * ESCA-JAVA0029:
    * @return
    */
   private int getAudioBufferSize() {return audioBufferSize;}
   
   
   
   /**
    * Gets the audio byte array.
    * @return
    */
   private byte[] getAudioByteArray() {return (audioByteArray);}
   
   
   
   /**
    * Gets the audio byte array length.
    * @return
    */
   private int getAudioByteArrayLength() {return (audioByteArray.length);}
   
   
   
   /**
    * Gets the video byte array.
    * @return
    */
   private byte[] getVideoByteArray() {return (videoStream.toByteArray());}
   
     
   //***************************************** END PRIVATE METHODS *****************************************//
   
   
   private byte[] data;
   private Webcam webcam;
   private float videoQuality;
   private BufferedImage image;
   private byte[] audioByteArray;
   private Thread frameRefreshing;
   private ImageIcon videoThumbnail;
   private String messageId, channelId;
   private TargetDataLine targetDataLine;
   private SourceDataLine sourceDataLine;
   private int audioBufferSize, webcamCamera;
   private ByteArrayOutputStream videoStream;
   private DataUploadDownload dataUploadDownload;

   ArrayList<Integer> indexes = new ArrayList<Integer>();
   
   private ArrayList<String> writeSettings, readSettings;
   
   ArrayList<byte[]> messageBytes = new ArrayList<byte[]>();

   private boolean stopRecord, stopPlayback, stopRefresh, modeRecord, modePlayback;
   private int audioChannels, audioBytes, videoWidth, videoHeight, frameRate, audioSamples, currentSize;
}


//********************************************** END CLASS **********************************************//