/*
   Copyright (c) 2010 - 2013 Javaika.com by Alexander Wait.
   
   All Rights Reserved.
*/



package required;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;
import javax.swing.ImageIcon;

import main.OnlineMessagingGui;


//********************************************* START CLASS *********************************************//


/**
 * This class receives audio/video from a transmitter over TCP-IP.
 * 
 * @author Alexander Wait
 * @version 2014-07-25
 */
public class AudioVideoReceive
{
   /* Configuration Settings */

   private static int PORT_POLLING_DELAY = 500;
   
   private static int AUDIO_MSG_SIZE = 20;

   private static int PACKET_SIZE = 1000;
   
   private static int INFO_SIZE = 100;

  
   /**
    * Class constructor.
    * @param ipAddress
    * @param audioDataPort
    * @param videoDataPort
    * @param protocol
    * ESCA-JAVA0265:
    * ESCA-JAVA0266:
    */
   public AudioVideoReceive(String protocol, String ipAddress, int audioDataPort, int videoDataPort)
   {
      this.protocol = protocol; this.ipAddress = ipAddress;
      
      this.audioDataPort = audioDataPort; this.videoDataPort = videoDataPort;
      
      
      videoThumbnail = new ImageIcon(this.getClass().getResource
      
      ("/files/images/VideoMessagingImageBlank.png"));
   }
   
   
   //**************************************** START PUBLIC METHODS *****************************************//
   
   
   /**
    * Starts the reception.
    */
   public void receive() {startAudioReceptionThread();}
   
   
   
   /**
    * Stops the reception.
    * ESCA-JAVA0266:
    * ESCA-JAVA0166:
    */
   public void cancel()
   { 
      try
      {
         if (sourceDataLine != null)
         { 
            if (sourceDataLine.isOpen()) {sourceDataLine.close();}
         }
         
         
         if (audioDataSocket != null)
         { 
            if (audioDataSocket.isBound()) {audioDataSocket.close();}
         }
         
         
         if (videoDataSocket != null)
         { 
            if (videoDataSocket.isBound()) {videoDataSocket.close();}
         }
         
         
         if (audioDatagramSocket != null)
         { 
            if (audioDatagramSocket.isBound()) {audioDatagramSocket.close();}
         }
         
         
         if (videoDatagramSocket != null)
         { 
            if (videoDatagramSocket.isBound()) {videoDatagramSocket.close();}
         }
      }
      
      catch (IOException e) {System.out.println(e);} 
   }
   
   
   
   /**
    * Sets the reception boolean variable.
    * @param value
    */
   public void setStopReceive(boolean value) {stopReceive = value;}
   
   
   //***************************************** END PUBLIC METHODS ******************************************//
   
   
   //**************************************** START PRIVATE METHODS ****************************************//
   

   private class StartAudioDataReception extends Thread 
   {

      /** 
       * Thread that starts the audio data reception.
       */
      private StartAudioDataReception() 
      {
         setDaemon(false);
      }
  
      
      /**
       * ESCA-JAVA0166:
       * ESCA-JAVA0266:
       */
      public void run()
      {
         if (protocol.equals("TCP")) 
         {
            receiveTCPAudioData();
         }
         
         
         if (protocol.equals("UDP"))
         {
            receiveUDPAudioData();
         }
      }
   }
   
   
   
   private class StartVideoDataReception extends Thread 
   {

      /** 
       * Thread that starts the video data reception.
       */
      private StartVideoDataReception() 
      {
         setDaemon(false);
      }
  
      
      /**
       * ESCA-JAVA0166:
       * ESCA-JAVA0266:
       */
      public void run()
      {
         if (protocol.equals("TCP"))
         {
            receiveTCPVideoData();
         }
         
         
         if (protocol.equals("UDP"))
         {
            receiveUDPVideoData();
         }
      }
   }
   
   
   
   /**
    * Starts the audio reception thread.
    * ESCA-JAVA0087:
    * ESCA-JAVA0266:
    */
   public void startAudioReceptionThread()
   {
      audioDataReceive = new StartAudioDataReception(); 
      
      audioDataReceive.start();
   }
   
   
   
   /**
    * Starts the video reception thread.
    * ESCA-JAVA0087:
    * ESCA-JAVA0266:
    */
   public void startVideoReceptionThread()
   {
      videoDataReceive = new StartVideoDataReception(); 
      
      videoDataReceive.start();
   }
   
   
   
   /**
    * Starts the audio data-gram socket.
    * ESCA-JAVA0266:
    * ESCA-JAVA0087:
    * ESCA-JAVA0166:
    */
   private void audioDataDatagramSocketStart()
   {
      try 
      {
         audioDatagramSocket = new DatagramSocket(audioDataPort);
         
         // System.out.println("Audio Receive Local Port: " + audioDatagramSocket.getLocalPort());
      }
      
      catch (IOException e) 
      {
         System.out.println("Error initialising audio data datagram socket.");
      }
   }
   
   
   
   /**
    * Starts the video data-gram socket.
    * ESCA-JAVA0266:
    * ESCA-JAVA0087:
    * ESCA-JAVA0166:
    */
   private void videoDataDatagramSocketStart()
   {
      try 
      {
         videoDatagramSocket = new DatagramSocket(videoDataPort);
         
         // System.out.println("Video Receive Local Port: " + videoDatagramSocket.getLocalPort());
      }
      
      catch (IOException e) 
      {
         System.out.println("Error initialising video data datagram socket.");
      }
   }
   
   
   
   /**
    * Connects to the audio data socket.
    * ESCA-JAVA0266:
    * ESCA-JAVA0087:
    * ESCA-JAVA0166:
    */
   private void audioDataSocketConnect()
   {
      try 
      {
         Thread.sleep(PORT_POLLING_DELAY); // Wait half a second until next poll.
      
         audioDataSocket = new Socket(ipAddress, audioDataPort);
      }
      
      catch (IOException | InterruptedException e) 
      {
         audioDataSocketConnect(); // Recursive call to instantiate audio socket.
      }
   }
   
   
   
   /**
    * Connects to the video data socket.
    * ESCA-JAVA0266:
    * ESCA-JAVA0087:
    * ESCA-JAVA0166:
    */
   private void videoDataSocketConnect()
   {
      try 
      {
         Thread.sleep(PORT_POLLING_DELAY); // Wait half a second until next poll.
      
         videoDataSocket = new Socket(ipAddress, videoDataPort);
      }
      
      catch (IOException | InterruptedException e) 
      {
         videoDataSocketConnect(); // Recursive call to instantiate video socket.
      }
   }
   
   
   
   /**
    * Receives TCP message from server.
    * ESCA-JAVA0266:
    * ESCA-JAVA0166:
    * ESCA-JAVA0029:
    */
   private void receiveTCPMessage()
   { 
      try
      {
         audioDataSocketConnect();
         
         
         audioBufferedReader = new BufferedReader
         (
            new InputStreamReader(audioDataSocket.getInputStream())
         );
         

         String temp = audioBufferedReader.readLine();
         
         if (temp != null) {setMessage(temp);}
      }
   
      catch (Exception e) 
      {
         System.out.println("An error has occured receiving message.");
      }
   }
   
   
   
   /**
    * Receives the TCP audio data from the server.
    * ESCA-JAVA0266:
    * ESCA-JAVA0166:
    * ESCA-JAVA0087:
    */
   public void receiveTCPAudioData()
   { 
      try
      {
         receiveTCPMessage(); startVideoReceptionThread();
         
         setAudioFormat(getMessage()); startSourceLine();
   
         
         ByteArrayOutputStream audioByteStream = new ByteArrayOutputStream();
         

         while (!getStopReceive())
         {
            if ((audioReadData = audioBufferedReader.readLine()) != null)
            {
               audioByteStream.reset(); list = Arrays.asList
               (
                  audioReadData.substring(1, audioReadData.length() - 1).split(", ")
               );
            

               for (String element : list) 
               {
                  audioByteStream.write(Byte.valueOf(element));
               }

               
               audioData = audioByteStream.toByteArray();
               
               writeAudio(); audioByteStream.flush();
            }
         }
      }
      
      catch (NullPointerException | IOException e)
      {
         System.out.println("An error has occured reading audio data.");
      }
      
      finally
      {
         setMessage(null); // Resets the message.
      }
   }
   
   
   
   /**
    * Receives the TCP video data from the server.
    * ESCA-JAVA0266:
    * ESCA-JAVA0166:
    * ESCA-JAVA0087:
    */
   public void receiveTCPVideoData()
   { 
      try
      {
         videoDataSocketConnect(); byte[] chunk = new byte[PACKET_SIZE];
         
         
         videoBufferedReader = new BufferedReader
         (
            new InputStreamReader(videoDataSocket.getInputStream())
         );
         
   
         ByteArrayOutputStream videoByteStream = new ByteArrayOutputStream();
       
         
         while (!getStopReceive())
         {
            if ((videoReadData = videoBufferedReader.readLine()) != null)
            {
               list = Arrays.asList
               (
                  videoReadData.substring(1, videoReadData.length() - 1).split(", ")
               );
         
               
               for (int i = 0; i < chunk.length; i ++) {chunk[i] = Byte.valueOf(list.get(i));}
            
               
               if (isByteArrayEmpty(chunk))
               { 
                  videoData = videoByteStream.toByteArray(); writeVideo();

                  videoByteStream.flush(); videoByteStream.reset(); 
               }
           
               else
               {
                  videoByteStream.write(chunk);
               }
            }
         }
      }
      
      catch (NullPointerException | IOException e)
      {
         System.out.println("An error has occured reading data.");
      }
      
      finally
      {
         OnlineMessagingGui.clearLabelIcon(videoThumbnail); // Reset video view port.
      }
   }
   
   
   
   /**
    * Receives the UDP audio data from the server.
    * ESCA-JAVA0266:
    * ESCA-JAVA0166:
    * ESCA-JAVA0087:
    */
   public void receiveUDPAudioData()
   { 
      try
      {
         audioDataDatagramSocketStart(); startVideoReceptionThread();
         
         setAudioDatagramPacket(PACKET_SIZE);
         

         while (!getStopReceive())
         {
            audioDatagramSocket.receive(audioDatagramPacket);

            audioData = audioDatagramPacket.getData();
 
            writeAudio();
         }
      }
      
      catch (NullPointerException | IOException e)
      {
         System.out.println("An error has occured reading audio data.");
      }
      
      finally
      {
         setMessage(null); // Resets the message.
      }
   }
   
   
   
   /**
    * Receives the UDP video data from the server.
    * ESCA-JAVA0266:
    * ESCA-JAVA0166:
    * ESCA-JAVA0087:
    */
   public void receiveUDPVideoData()
   { 
      try
      {
         videoDataDatagramSocketStart(); setVideoDatagramPacket(PACKET_SIZE);   
         
         ByteArrayOutputStream videoByteStream = new ByteArrayOutputStream();
      

         while (!getStopReceive())
         {  
            videoDatagramSocket.receive(videoDatagramPacket);
             
            
            if (isByteArrayEmpty(udpVideoReceiveArray))
            {
               if (isFrameComplete())
               {
                  for (int i = 0; i < frameBytes.size(); i++)
                  {
                     videoByteStream.write(Arrays.copyOfRange(frameBytes.get(i), INFO_SIZE, PACKET_SIZE));
                  }
            
                  
                  videoData = videoByteStream.toByteArray(); writeVideo();
               
                  videoByteStream.flush(); videoByteStream.reset();
               }

            
               frameBytes.clear();
            }
            
            
            else
            {
               frameBytes.add(Arrays.copyOfRange(udpVideoReceiveArray, 0, PACKET_SIZE));
            }  
         } 
      }
      
      catch (NullPointerException | IOException e)
      {
         System.out.println("An error has occured reading video data.");
      }
      
      finally
      {
         OnlineMessagingGui.clearLabelIcon(videoThumbnail); // Reset video view port.
      }
   }
   
   
   
   /**
    * Tests the chunk integrity to render a frame.
    * @return
    */
   private boolean isFrameComplete()
   {
      boolean complete = true;

      
      for (int i = 1; i < frameBytes.size(); i ++)
      {
         byte[] infoA = Arrays.copyOfRange(frameBytes.get(i -1), 0, INFO_SIZE);
         
         byte[] infoB = Arrays.copyOfRange(frameBytes.get(i), 0, INFO_SIZE);

         
         String[] partsA = new String(infoA).trim().split("-");
         
         String[] partsB = new String(infoB).trim().split("-");
         
         
         if ((Integer.parseInt(partsA[1])) != (Integer.parseInt(partsB[1]) -1))
         {
            complete = false; // Test if chunks are in correct order.
         }
         
         
         if ((i == (frameBytes.size() - 1)) && (!partsB[1].equals(partsB[2])))
         {
            complete = false; // Test if all the chunks are present.
         }
         
         
         if ((!partsA[0].equals(partsB[0])))
         {
            complete = false; // Test if chunks are from same frame.
         }
      }
      
      
      return complete;
   }
   
   
   
   /**
    * Checks if a byte array is empty.
    * @param bytes
    * @return
    */
   private static boolean isByteArrayEmpty(byte[] bytes)
   {
      for (byte b : bytes)
      {
         if (b != 0) {return false;}
      }
      
      return true;
   }
   


   /**
    * Trim a byte array with trailing null values.
    * ESCA-JAVA0029:
    * @param bytes
    * @return
    */
   @SuppressWarnings("unused")
   private static byte[] trimByteArray(byte[] bytes)
   {
       int i = bytes.length - 1;
       
       while ((bytes[i] == 0) && (i >= 0)) {i --;}
       
      
       return Arrays.copyOf(bytes, i + 1);
   }
   
   
   
   /**
    * Writes the audio data.
    */
   private void writeAudio()
   {
      setAudioByteArray(audioData);  
   
   
      if (protocol.equals("TCP"))
      {
         sourceDataLine.write(getAudioByteArray(), 0, getAudioByteArrayLength());
      }
   
   
      if (protocol.equals("UDP"))
      {  
         if (!audioConfigRead)
         {
            setMessage(new String(Arrays.copyOfRange(audioData, 0, AUDIO_MSG_SIZE - 1)).toString());

            setAudioFormat(getMessage()); startSourceLine(); audioConfigRead = true;
         }
      
      
         else
         {
            sourceDataLine.write(getAudioByteArray(), AUDIO_MSG_SIZE, getAudioByteArrayLength() - AUDIO_MSG_SIZE);
         }
      } 
   }
   
   
   
   /**
    * Writes the video data.
    */
   private void writeVideo()
   {
      setVideoByteArray(videoData);

      OnlineMessagingGui.setLabelIcon(getVideoByteArray());
   }
   


   /**
    * Starts the source data line.
    * ESCA-JAVA0266:
    * ESCA-JAVA0166:
    */
   private void startSourceLine()
   {
      DataLine.Info dataLineInfo = new DataLine.Info(SourceDataLine.class, getAudioFormat());
 
      
      try
      {      
         sourceDataLine = (SourceDataLine) AudioSystem.getLine(dataLineInfo);
      
         sourceDataLine.open(getAudioFormat());
         
         sourceDataLine.start();
      }
      
      catch (LineUnavailableException | IllegalArgumentException e)
      {
         System.out.println("The audio system line is unavailable.");
      }
   }
   
   
   
   /**
    * Extracts and sets the audio format.
    * ESCA-JAVA0166:
    * ESCA-JAVA0266:
    * @param format
    */
   private void setAudioFormat(String format)
   {
      try
      {   
         // Split the string into individual components.
   
         String[] parts = (format.trim().split(","));
      
      
         // Initialize audio format settings.
      
         samples = (float)Double.parseDouble(parts[0]);

         channels = Integer.parseInt(parts[1]);
      
         bytes = Integer.parseInt(parts[2]);
      }
      
      catch (NullPointerException e)
      { 
         System.out.println("Error extracting audio settings.");
      }
   }
   
   
   
   /**
    * Sets the audio data-gram packet with a specified size.
    * @param packetSize
    */
   private void setAudioDatagramPacket(int packetSize)
   {
      udpAudioReceiveArray = new byte[packetSize];

      audioDatagramPacket = new DatagramPacket(udpAudioReceiveArray, udpAudioReceiveArray.length);
   }
   
   
   
   /**
    * Sets the audio data-gram packet with a specified size.
    * @param packetSize
    */
   private void setVideoDatagramPacket(int packetSize)
   {
      udpVideoReceiveArray = new byte[packetSize];

      videoDatagramPacket = new DatagramPacket(udpVideoReceiveArray, udpVideoReceiveArray.length);
   }
   
   
   
   /**
    * Returns the audio format.
    * @return
    */
   private AudioFormat getAudioFormat()
   {
      boolean signed = true; boolean bigEndian = false;
      

      return new AudioFormat(samples, (8 * bytes), channels, signed, bigEndian);
   }
   
   
   
   /**
    * Sets the message.
    * @param val
    */
   private void setMessage(String val) {message = val;}
   
   
   
   /**
    * Sets the audio byte array.
    * @param array
    */
   private void setAudioByteArray(byte[] array) {audioByteArray = array.clone();}
   
   
   
   /**
    * Sets the video byte array.
    * @param array
    */
   private void setVideoByteArray(byte[] array) {videoByteArray = array.clone();}
   

   
   /**
    * Gets the message.
    * @return
    */
   private String getMessage() {return message;}
   
   
   
   /**
    * Returns the reception boolean variable.
    * @return
    */
   private boolean getStopReceive() {return stopReceive;}
   
   
   
   /**
    * Gets the audio byte array.
    * @return
    */
   private byte[] getAudioByteArray() {return (audioByteArray);}
   
   
   
   /**
    * Gets the video byte array.
    * @return
    */
   private byte[] getVideoByteArray() {return (videoByteArray);}
   
   
   
   /**
    * Gets the audio byte array length.
    * @return
    */
   private int getAudioByteArrayLength() {return (audioByteArray.length);}
   

   //***************************************** END PRIVATE METHODS *****************************************//


   private float samples;
   private ImageIcon videoThumbnail;
   private SourceDataLine sourceDataLine;
   private boolean audioConfigRead, stopReceive;
   private Socket audioDataSocket, videoDataSocket;
   private Thread audioDataReceive, videoDataReceive;
   private int audioDataPort, videoDataPort, channels, bytes;
   private DatagramSocket audioDatagramSocket, videoDatagramSocket;
   private DatagramPacket audioDatagramPacket, videoDatagramPacket;
   private BufferedReader audioBufferedReader, videoBufferedReader;
   private String protocol, message, ipAddress, audioReadData, videoReadData;
   private byte[] udpAudioReceiveArray, udpVideoReceiveArray, audioByteArray, videoByteArray, audioData, videoData;
   
   ArrayList<byte[]> frameBytes = new ArrayList<byte[]>(); List<String> list = new ArrayList<String>();
}


//********************************************** END CLASS **********************************************//