/*
   Copyright (c) 2010 - 2013 Javaika.com by Alexander Wait.
   
   All Rights Reserved.
*/



package required;

import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.net.*;
import java.util.Arrays;
import java.util.Iterator;

import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageWriter;
import javax.imageio.stream.ImageOutputStream;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.Port;
import javax.sound.sampled.TargetDataLine;

import com.github.sarxos.webcam.Webcam;

import main.OnlineMessagingGui;


//********************************************* START CLASS *********************************************//


/**
 *  This class transmits audio/video to a receiver over TCP-IP.
 *
 *  @author Alexander Wait
 *  @version 2014-07-25
 */
public class AudioVideoTransmit
{
   /* Configuration Settings */

   private static boolean VIDEO = false;
   
   
   private static int AUDIO_MSG_SIZE = 20;
   
   private static int PACKET_SIZE = 1000;
   
   private static int INFO_SIZE = 100;
   
   
   /**
    * Class constructor.
    * @param protocol
    * @param ipAddress
    * @param audioDataPort
    * @param videoDataPort
    * ESCA-JAVA0265:
    * ESCA-JAVA0266:
    */
   public AudioVideoTransmit(String protocol, String ipAddress, int audioDataPort, int videoDataPort) 
   {
      this.protocol = protocol; this.ipAddress = ipAddress;
      
      this.audioDataPort = audioDataPort; this.videoDataPort = videoDataPort;
      
      
      audioMessage = new byte[AUDIO_MSG_SIZE]; videoStream = new ByteArrayOutputStream();
   }
   
   
   //**************************************** START PUBLIC METHODS *****************************************//
   
   
   /**
    * Starts the transmission.
    * ESCA-JAVA0266:
    */
   public void transmit() {setBufferArray(); transmissionCheck();}
   
   
   
   /**
    * Stops the transmission.
    * ESCA-JAVA0266:
    * ESCA-JAVA0166:
    */
   public void cancel()
   { 
      try
      { 
         if (targetDataLine != null)  
         { 
            if (targetDataLine.isOpen()) {targetDataLine.close();}
         }
         
         if (audioDataServerSocket != null)
         {
            if (audioDataServerSocket.isBound()) {audioDataServerSocket.close();}
         }
         
         if (videoDataServerSocket != null)
         {
            if (videoDataServerSocket.isBound()) {videoDataServerSocket.close();}
         }
         
         if (audioDataSocket != null)
         { 
            if (audioDataSocket.isBound()) {audioDataSocket.close();}
         }
         
         if (videoDataSocket != null)
         { 
            if (videoDataSocket.isBound()) {videoDataSocket.close();}
         }

         if (videoStream != null) {videoStream.close();}
         
         if (webcam != null) {webcam.close();}
      }
      
      catch (IOException e) {System.out.println(e);} 
   }
   
   
   
   /**
    * Sets the video inclusion boolean variable.
    * @param value
    */
   public void setIncludeVideo(boolean value) {VIDEO = value;}
   
   
   
   /**
    * Sets the audio samples.
    * @param val
    */
   public void setAudioSamples(float val) {audioSamples = val;}
   
   
   
   /**
    * Sets the frame rate.
    * @param val
    */
   public void setFrameRate(int val) {frameRate = val;}
   
   
   
   /**
    * Sets the video quality.
    * @param val
    */
   public void setVideoQuality(float val) {videoQuality = val;}
   
   
   
   /**
    * Sets the audio channels.
    * @param val
    */
   public void setAudioChannels(int val) {audioChannels = val;}
   
   
   
   /**
    * Sets the audio chunk factor.
    * @param val
    */
   public void setAudioChunkFactor(int val) {audioChunkFactor = val;}
   
   
   
   /**
    * Sets the audio byte number.
    * @param val
    */
   public void setAudioBytes(int val) {audioBytes = val;}
   
   
   
   /**
    * Sets the video width.
    * @param val
    */
   public void setVideoWidth(int val) {videoWidth = val;}
   
   
   
   /**
    * Sets the video height.
    * @param val
    */
   public void setVideoHeight(int val) {videoHeight = val;}
   
   
   
   /**
    * Sets the transmission boolean variable.
    * @param value
    */
   public void setStopTransmit(boolean value) {stopTransmit = value;}
   
   
   
   /**
    * Sets the camera variable.
    * @param value
    */
   public void setWebcamCamera(int value) {webcamCamera = value;}
   
   
   //***************************************** END PUBLIC METHODS ******************************************//
    
   
   //**************************************** START PRIVATE METHODS ****************************************//
   
   
   private class StartAudioDataTransmission extends Thread 
   {

      /** 
       * Thread that starts the data transmission.
       */
      private StartAudioDataTransmission() 
      {
         setDaemon(false);
      }
       
      
      /**
       * ESCA-JAVA0166:
       * ESCA-JAVA0266:
       */
      public void run()
      {    
         if (protocol.equals("TCP")) 
         {
            transmitTCPAudioData();
         }
 
         
         if (protocol.equals("UDP"))
         {
            transmitUDPAudioData();
         }
      }
   }
   
   
   
   private class StartVideoDataTransmission extends Thread 
   {

      /** 
       * Thread that starts the data transmission.
       */
      private StartVideoDataTransmission() 
      {
         setDaemon(false);
      }
       
      
      /**
       * ESCA-JAVA0166:
       * ESCA-JAVA0266:
       */
      public void run()
      {
         if (protocol.equals("TCP"))
         {
            transmitTCPVideoData();
         }
         
         
         if (protocol.equals("UDP"))
         {
            transmitUDPVideoData();
         }
      }
   }
   
   
   
   /**
    * Starts the audio transmission thread.
    * ESCA-JAVA0087:
    * ESCA-JAVA0266:
    */
   private void startAudioTransmissionThread()
   {
      audioDataTransmit = new StartAudioDataTransmission();
      
      audioDataTransmit.start();
   }
   
   
   
   /**
    * Starts the video transmission thread.
    * ESCA-JAVA0087:
    * ESCA-JAVA0266:
    */
   private void startVideoTransmissionThread()
   {
      videoDataTransmit = new StartVideoDataTransmission();
      
      videoDataTransmit.start();
   }
   
   
   
   /**
    * Starts data transmission if audio can be at least transmitted. 
    * ESCA-JAVA0087:
    * ESCA-JAVA0266:
    */
   private void transmissionCheck()
   {
      if (isMicrophoneAvailable())
      { 
         startTargetLine(); // Starts the target data line if microphone is available.
   
         
         if (getIncludeVideo()) 
         {
            startWebcam(); // Attempts to start the web camera if enabled.
         } 
         
         else
         {
            OnlineMessagingGui.appendTextOutput("\n Camera checkbox is unselected.");
          
            OnlineMessagingGui.appendTextOutput("\n Video transmission is disabled.");
         }
         
         
         audioTransmissionCheck();
      }
      
      else
      {
         OnlineMessagingGui.appendTextOutput("\n A microphone is not available.");
          
         OnlineMessagingGui.appendTextOutput("\n Audio can not be transmitted.");
      }
   }
   
   
   
   /**
    * Check if audio can be transmitted.
    */
   private void audioTransmissionCheck()
   {
      if (targetDataLine != null)
      {
         if (protocol.equals("TCP")) {audioDataServerSocketStart();}
      
         if (protocol.equals("UDP")) {audioDataDatagramSocketStart();}
          

         startAudioTransmissionThread();
      }
       
      else
      {
         OnlineMessagingGui.appendTextOutput("\n Microphone could not be loaded.");
           
         OnlineMessagingGui.appendTextOutput("\n Audio can not be transmitted.");
      }   
   }
   
   
   
   /**
    * Check if video can be transmitted.
    */
   private void videoTransmissionCheck()
   {
      if (getIncludeVideo() && (webcam != null))
      {
         if (protocol.equals("TCP")) {videoDataServerSocketStart();}
          
         if (protocol.equals("UDP")) {videoDataDatagramSocketStart();}
          
         
         startVideoTransmissionThread();
      }  
       
      else
      {
         OnlineMessagingGui.appendTextOutput("\n Camera could not be started.");
    
         OnlineMessagingGui.appendTextOutput("\n Video can not be transmitted.");
      }
   }
   
   
   
   /**
    * Starts the web camera.
    * ESCA-JAVA0266:
    */
   private void startWebcam()
   {
      webcam = Webcam.getWebcams().get(getWebcamCamera());
      
      webcam.setCustomViewSizes(new Dimension[] {new Dimension(getVideoWidth(), getVideoHeight())});

      webcam.setViewSize(new Dimension(getVideoWidth(), getVideoHeight())); webcam.open();
   }

   
   
   /**
    * Sets the audio buffer array.
    */
   private void setBufferArray()
   {
      final int MIN_CHUNK_SIZE = 600; final int MULTIPLY_FACTOR = 8;
      
      int testSize = (int)Math.floor(MIN_CHUNK_SIZE + MULTIPLY_FACTOR * (getAudioChunkFactor())); 
      
      
      int frameSize = getAudioBytes() * getAudioChannels();
      
     
      for (int i = 0; i < frameSize; i ++)
      {
         if ((testSize % frameSize) == 0)
         {
            audioBufferSize = testSize; break;
         }
         
         else {testSize --;}
      }
      

      audioByteArray = new byte[getAudioBufferSize()];  
   }
   
   

   /**
    * Starts the audio data server socket.
    * ESCA-JAVA0266:
    */
   private void audioDataServerSocketStart()
   {
      try
      {
         audioDataServerSocket = new ServerSocket(audioDataPort);
      }
      
      catch (IOException e)
      {
         System.out.println("Error initialising audio data server socket.");
      }
   }
   
   
   
   /**
    * Starts the audio data-gram socket.
    * ESCA-JAVA0266:
    */
   private void audioDataDatagramSocketStart()
   {
      try
      {
         audioDatagramSocket = new DatagramSocket();
         
         // System.out.println("Audio Transmit Local Port: " + audioDatagramSocket.getLocalPort());

         transmitHost = InetAddress.getByName(ipAddress); 
      }
      
      catch (IOException e)
      {
         System.out.println("Error initialising audio data datagram socket.");
      }
   }
   
   
   
   /**
    * Starts the video data server socket.
    * ESCA-JAVA0266:
    */
   private void videoDataServerSocketStart()
   {
      try
      {
         videoDataServerSocket = new ServerSocket(videoDataPort);
      }
      
      catch (IOException e)
      {
         System.out.println("Error initialising video data server socket.");
      }
   }
   
   
   
   /**
    * Starts the video data-gram socket.
    * ESCA-JAVA0266:
    */
   private void videoDataDatagramSocketStart()
   {
      try
      {
         videoDatagramSocket = new DatagramSocket();
         
         // System.out.println("Video Transmit Local Port: " + videoDatagramSocket.getLocalPort());
        
         transmitHost = InetAddress.getByName(ipAddress); 
      }
      
      catch (IOException e)
      {
         System.out.println("Error initialising video data datagram socket.");
      }
   }


   
   /**
    * Sends text output to client via TCP.
    * @param msg
    * ESCA-JAVA0266:
    * ESCA-JAVA0029:
    * ESCA-JAVA0166:
    */
   private void transmitTCPMessage(String msg)
   {
      try
      {
         audioDataSocket = audioDataServerSocket.accept();
       
         
         audioPrintStream = new PrintStream(audioDataSocket.getOutputStream());
         
         audioPrintStream.println(msg);
      }
   
      catch (Exception e)
      {
         System.out.println("Error running text output server module.");
      }
   }
   
   
   
   /**
    * Transmits the TCP audio data to the client.
    * ESCA-JAVA0166:
    * ESCA-JAVA0266:
    */
   private void transmitTCPAudioData()
   {
      try
      { 
         transmitTCPMessage // Send audio configuration information to the receiver for decode.
         (
            getAudioSamples()+","+getAudioChannels()+","+getAudioBytes()
         );
         
         
         videoTransmissionCheck(); // Include video transmission if enabled.

         
         while (!getStopTransmit())
         {
            readAudio(); writeAudioData = Arrays.toString(audioData);
          
            audioPrintStream.println(writeAudioData);
         }
      }
      
      catch (NullPointerException e)
      {
         System.out.println("An error has occured writing audio data.");
      }
   }
   
   
   
   /**
    * Transmits the TCP video data to the client.
    * ESCA-JAVA0087:
    * ESCA-JAVA0166:
    * ESCA-JAVA0266:
    */
   private void transmitTCPVideoData()
   {
      try
      {
         videoDataSocket = videoDataServerSocket.accept(); int iStart = 0, iEnd = 0;
         
         videoPrintStream = new PrintStream(videoDataSocket.getOutputStream());

         
         while (!getStopTransmit())
         {
            readVideo(); int chunksInFrame = (int)Math.ceil
            (
               ((double)videoData.length / (double)PACKET_SIZE)
            );
            
            
            for (int i = 0; i < chunksInFrame; i ++)
            {
               iStart = i * PACKET_SIZE; iEnd = iStart + PACKET_SIZE;
               
               
               if (iEnd < videoData.length)
               {
                  byte[] chunk = Arrays.copyOfRange(videoData, iStart, iEnd);
    
                  writeVideoData = Arrays.toString(chunk);
               }
                    
               else
               {
                  byte[] chunk = returnAdditionalBytes(videoData, iStart, videoData.length);
               
                  writeVideoData = Arrays.toString(chunk); 
               }
               
               videoPrintStream.println(writeVideoData);  
            }
            
            
            writeVideoData = Arrays.toString(new byte[PACKET_SIZE]);
       
            videoPrintStream.println(writeVideoData);
 
            Thread.sleep(getFrameRate());
         }
      }
      
      catch (InterruptedException | NullPointerException | IOException e)
      {
         System.out.println("An error has occured writing video data.");
      }
   }
   
   
   
   /**
    * Transmits the UDP audio data to the client.
    * ESCA-JAVA0166:
    * ESCA-JAVA0266:
    */
   private void transmitUDPAudioData()
   {
      try
      { 
         setMessage(getAudioSamples()+","+getAudioChannels()+","+getAudioBytes());
     
         setAudioMessageBytes(getMessage()); setAudioDatagramPacket(PACKET_SIZE);
         
         videoTransmissionCheck(); // Include video transmission if enabled.
         

         while (!getStopTransmit())
         {
            readAudio(); System.arraycopy(audioData, 0, udpAudioTransmitArray, 0, audioData.length);
            
            audioDatagramSocket.send(audioDatagramPacket);
         }
      }
      
      catch (NullPointerException | IOException e)
      {
         System.out.println("An error has occured writing audio data.");
      }
   }
   
   
   
   /**
    * Transmits the UDP video data to the client.
    * ESCA-JAVA0087:
    * ESCA-JAVA0166:
    * ESCA-JAVA0266:
    */
   private void transmitUDPVideoData()
   {
      try
      {
         setVideoDatagramPacket(PACKET_SIZE); 
         
         int frame = 0, chunks = 0, iStart = 0, iEnd = 0;
         
         
         while (!getStopTransmit())
         {
            readVideo(); chunks = (int)Math.ceil
            (
               ((double)videoData.length / ((double)(PACKET_SIZE - INFO_SIZE)))
            );
            
            
            for (int i = 0; i < chunks; i ++)
            { 
               iStart = i * (PACKET_SIZE - INFO_SIZE); iEnd = iStart + PACKET_SIZE - INFO_SIZE;
            
               byte[] bytes = Arrays.copyOfRange(videoData, iStart, iEnd);

               byte[] info = getInfoBytes(frame, i, chunks);
              
              
               System.arraycopy(info, 0, udpVideoTransmitArray, 0, info.length);
               
               System.arraycopy(bytes, 0, udpVideoTransmitArray, info.length, bytes.length);
               
               
               videoDatagramSocket.send(videoDatagramPacket);
            }
            
            
            clearVideoTransmitArray(); videoDatagramSocket.send(videoDatagramPacket); frame ++;
            
            Thread.sleep(getFrameRate());
         }
      }
      
      catch (InterruptedException | NullPointerException | IOException e)
      {
         System.out.println("An error has occured writing video data.");
      }
   }
   
   
   
   /**
    * Returns the frame info bytes.
    * @param frameIndex
    * @param chunkIndex
    * @param chunkTotal
    * @return
    */
   private static byte[] getInfoBytes(int frameIndex, int chunkIndex, int chunkTotal)
   {
      byte[] arrayBytes = new byte[INFO_SIZE];
      
      byte[] infoBytes = ((frameIndex + 1) + "-" + (chunkIndex + 1) + "-" + chunkTotal).getBytes();
      
      
      System.arraycopy(infoBytes, 0, arrayBytes, 0, infoBytes.length);

      
      return arrayBytes;
   }
   
   

   /**
    * Returns additional bytes to the TCP video transmission.
    * @param array
    * @param startIndex
    * @param endIndex
    * @return
    */
   private static byte[] returnAdditionalBytes(byte[] array, int startIndex, int endIndex)
   {
      int len = endIndex - startIndex; byte[] chunk = new byte[PACKET_SIZE];


      for (int i = 0; i < chunk.length; i ++)
      {
         if (i < len)
         {
            chunk[i] = array[startIndex + i];
         }
         
         else
         {
            chunk[i] = 0;
         }
      }
      
      
      return chunk;
   }
   

   
   /**
    * Clears the video transmit array.
    */
   private void clearVideoTransmitArray()
   {
      for (int i = 0; i < udpVideoTransmitArray.length; i ++)
      {
          udpVideoTransmitArray[i] = 0;
      }
   }
   
   
   
   /**
    * Reads the audio data.
    */
   private void readAudio()
   {   
      if (protocol.equals("TCP"))
      {
         audioByteArray = new byte[getAudioBufferSize()]; 
       

         targetDataLine.read(getAudioByteArray(), 0, getAudioByteArrayLength()); 

         audioData = getAudioByteArray(); 
      }
      
      
      if (protocol.equals("UDP"))
      {
         audioByteArray = new byte[PACKET_SIZE]; 

         
         System.arraycopy(audioMessage, 0, getAudioByteArray(), 0, audioMessage.length);
          
          
         targetDataLine.read
         (
            getAudioByteArray(), audioMessage.length, getAudioByteArrayLength() - audioMessage.length
         ); 

         
         audioData = getAudioByteArray();
      }
   }
   
   
   
   /**
    * Reads the video data.
    */
   private void readVideo()
   { 
      if ((image = webcam.getImage()) != null)
      {   
         compressImage(image); videoData = getVideoByteArray();
         
         resetVideoStream();
      }
   }
   
   
   
   /**
    * Sets the audio message byte array.
    * @param msg
    */
   private void setAudioMessageBytes(String msg)
   {
      messageAsBytes = msg.getBytes();
      
      
      if (messageAsBytes.length <= audioMessage.length)
      {
         for (int i = 0; i < messageAsBytes.length; i ++)
         {
            audioMessage[i] = messageAsBytes[i];
         }
      }
   }
   
   
   
   /**
    * Resets the byte array output stream.
    */
   private void resetVideoStream()
   {
      videoStream.reset();
   }
   
   
   
   /**
    * Tests if a microphone is available.
    * @return
    */
   private static boolean isMicrophoneAvailable()
   {
      if (AudioSystem.isLineSupported(Port.Info.MICROPHONE))
      {
         return true;
      }
      
      else
      {
         return false;
      }
   }
   

   
   /**
    * Compresses the buffered image before storage.
    * ESCA-JAVA0266:
    * @param img
    */
   private void compressImage(BufferedImage img)
   {     
      try
      {          
         ByteArrayOutputStream baos = new ByteArrayOutputStream();
         
         ImageOutputStream ios = ImageIO.createImageOutputStream(baos);
        
         
         Iterator<ImageWriter> writers = ImageIO.getImageWritersByFormatName("jpg");
         
         ImageWriter writer = writers.next();
          
         
         ImageWriteParam param = writer.getDefaultWriteParam();
         
         
         param.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
         
         param.setCompressionQuality(getVideoQuality()); writer.setOutput(ios); 
         
         writer.write(null, new IIOImage(img, null, null), param);
             
         
         byte[] imgBytes = baos.toByteArray(); writer.dispose(); 
             
         getVideoStream().write(imgBytes, 0, imgBytes.length);         
      }
        
      catch (IOException e)
      {
         System.out.println("An error has occured during image compression.");
      }
   }
   
   
   
   /**
    * Starts the target data line.
    * ESCA-JAVA0266:
    */
   private void startTargetLine()
   {   
      DataLine.Info dataLineInfo = new DataLine.Info(TargetDataLine.class, getAudioFormat());
      
        
      try
      {
         targetDataLine = (TargetDataLine) AudioSystem.getLine(dataLineInfo);
         
         targetDataLine.open(getAudioFormat());
      }
         
      catch (LineUnavailableException e)
      { 
         System.out.println("The audio system line is unavailable.");
      }
         
         
      targetDataLine.start();         
   }
   
   
   
   /**
    * Sets the audio data-gram packet with a specified size.
    * @param packetSize
    */
   private void setAudioDatagramPacket(int packetSize)
   {
      udpAudioTransmitArray = new byte[packetSize];

      
      audioDatagramPacket = new DatagramPacket
      (
         udpAudioTransmitArray, udpAudioTransmitArray.length, transmitHost, audioDataPort
      );
   }
   
   
   
   /**
    * Sets the video data-gram packet with a specified size.
    * @param packetSize
    */
   private void setVideoDatagramPacket(int packetSize)
   {
      udpVideoTransmitArray = new byte[packetSize];

      
      videoDatagramPacket = new DatagramPacket
      (
         udpVideoTransmitArray, udpVideoTransmitArray.length, transmitHost, videoDataPort
      );
   }
   


   /**
    * Returns the audio format.
    * @return
    */
   private AudioFormat getAudioFormat()
   {
      boolean signed = true; boolean bigEndian = false;
      
      final int MULTIPLY_FACTOR = 8;
    
      
      return new AudioFormat
      (
         getAudioSamples(), (MULTIPLY_FACTOR * getAudioBytes()), 
         
         getAudioChannels(), signed, bigEndian
      );
   }
   
   
   
   /**
    * Sets the message.
    * @param val
    */
   private void setMessage(String val) {message = val;}
   
   
   
   /**
    * Gets the message.
    * @return
    */
   private String getMessage() {return message;}
   
   
   
   /**
    * Gets the video inclusion boolean variable.
    * @return
    */
   private static boolean getIncludeVideo() {return (VIDEO);}
   
   
   
   /**
    * Gets the audio samples.
    * @return samples
    */
   private float getAudioSamples() {return audioSamples;}
   
   
   
   /**
    * Gets the frame rate.
    * @return quality
    */
   private int getFrameRate() {return frameRate;}
   
   
   
   /**
    * Gets the audio quality.
    * @return quality
    */
   private float getVideoQuality() {return videoQuality;}
   
   
   
   /**
    * Gets the audio channels.
    * @return channels
    */
   private int getAudioChannels() {return audioChannels;}
   
   
  
   /**
    * Gets the audioChunkFactor.
    * @return frames
    */
   private int getAudioChunkFactor() {return audioChunkFactor;}
   
   
   
   /**
    * Gets the audio byte number.
    * @return bytes
    */
   private int getAudioBytes() {return audioBytes;}
   
   
   
   /**
    * Gets the video width.
    * @return width
    */
   private int getVideoWidth() {return videoWidth;}
   

   
   /**
    * Gets the video height.
    * @return width
    */
   private int getVideoHeight() {return videoHeight;}
   
   
   
   /**
    * Gets the audio buffer size.
    * ESCA-JAVA0029:
    * @return
    */
   private int getAudioBufferSize() {return audioBufferSize;}
   
   
   
   /**
    * Gets the video output stream.
    * @return
    */
   private ByteArrayOutputStream getVideoStream() {return (videoStream);}
   
   
   
   /**
    * Gets the camera to use for transmission.
    * @return
    */
   private int getWebcamCamera() {return (webcamCamera);}
   
   
   
   /**
    * Gets the transmission boolean variable.
    * @return
    */
   private boolean getStopTransmit() {return (stopTransmit);}
   
   

   /**
    * Gets the audio byte array.
    * @return
    */
   private byte[] getAudioByteArray() {return (audioByteArray);}
   
   
   
   /**
    * Gets the audio byte array length.
    * @return
    */
   private int getAudioByteArrayLength() {return (audioByteArray.length);}
   
   
   
   /**
    * Gets the video byte array.
    * @return
    */
   private byte[] getVideoByteArray() {return (videoStream.toByteArray());}
   
   
   //***************************************** END PRIVATE METHODS *****************************************//


   private Webcam webcam;
   private String message;
   private BufferedImage image;
   private boolean stopTransmit;
   private InetAddress transmitHost;
   private String protocol, ipAddress;
   private TargetDataLine targetDataLine;
   private float audioSamples, videoQuality;
   private ByteArrayOutputStream videoStream;
   private String writeAudioData, writeVideoData;
   private Socket audioDataSocket, videoDataSocket;
   private Thread audioDataTransmit, videoDataTransmit;
   private PrintStream audioPrintStream, videoPrintStream;
   private DatagramSocket audioDatagramSocket, videoDatagramSocket;
   private DatagramPacket audioDatagramPacket, videoDatagramPacket;
   private ServerSocket audioDataServerSocket, videoDataServerSocket;
   private int audioDataPort, videoDataPort, webcamCamera, audioBufferSize;
   private int audioChannels, audioBytes, videoWidth, videoHeight, frameRate, audioChunkFactor;
   private byte[] udpAudioTransmitArray, udpVideoTransmitArray, audioByteArray, audioData, videoData;
   
   private byte[] audioMessage, messageAsBytes;
}


//********************************************** END CLASS **********************************************//