/*
   Copyright (c) 2010 - 2013 Javaika.com by Alexander Wait.
   
   All Rights Reserved.
*/



package required;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLEncoder;
import java.util.ArrayList;


//********************************************* START CLASS *********************************************//


/**
 *  This class provides a bridge that connects java with PHP
 *  scripts on the server for writing information from the 
 *  java application to a remote database on the server. 
 *  The class also receives data from PHP server scripts
 *  upon request for this information.
 *
 *  @author Alexander Wait
 *  @version 2012-03-18
 */
public class Portal
{
   private Portal() {}
   
   
   //**************************************** START PUBLIC METHODS *****************************************//
   

   /**
    * Sends data to a specified PHP script address.
    * ESCA-JAVA0266:
    * @param address
    * @param userId
    * @param userPw
    * @param data
    */
   public static void javaToPhpGate(String address, String userId, String userPw, ArrayList<String> data) 
   {
      scriptAddress = address;


      try 
      {
         url = new URL(scriptAddress); conn = url.openConnection(); conn.setDoOutput(true); 
      

         dataSend = URLEncoder.encode("userId", "UTF-8") + "=" + URLEncoder.encode(userId, "UTF-8")

         + "&" + URLEncoder.encode("userPw", "UTF-8") + "=" + URLEncoder.encode(userPw, "UTF-8");
         
         
         if (data != null) {setVariableValues(data);} sendData(); 
         
         reader = new BufferedReader(new InputStreamReader(conn.getInputStream())); 


         while ((dataRecieve = reader.readLine()) != null) 
         {

            System.out.println(dataRecieve);
         }


         reader.close();
         
      }
      
      catch (IOException e) {System.out.println(Tools.thisPathAndLine() + e + "\n");}

   }
   
  
   
   /**
    * Receives data from a specified PHP script address.
    * ESCA-JAVA0266:
    * @param address
    * @param userId
    * @param userPw
    * @param data
    * @return
    */ 
   public static ArrayList<String> phpToJavaGate(String address, String userId, String userPw, ArrayList<String> data)
   { 
      scriptAddress = address; 

      ArrayList<String> output = new ArrayList<String>();
   

      try
      { 
         url = new URL(scriptAddress); conn = url.openConnection(); conn.setDoOutput(true);
     

         dataSend = URLEncoder.encode("userId", "UTF-8") + "=" + URLEncoder.encode(userId, "UTF-8")

         + "&" + URLEncoder.encode("userPw", "UTF-8") + "=" + URLEncoder.encode(userPw, "UTF-8");
         
         
         if (data != null) {setVariableValues(data);} sendData(); 
         
         reader = new BufferedReader(new InputStreamReader(conn.getInputStream()));
       
         
         while ((dataRecieve = reader.readLine()) != null) 
         {
            output.add(dataRecieve);
         }
        

         reader.close();
        
      }
      
      catch (IOException e) {System.out.println(Tools.thisPathAndLine() + e + "\n");}
      

      return output;
         
   }
   
   
   
   /**
    * Clears database records using a specified PHP script address.
    * ESCA-JAVA0266:
    * @param address
    * @param userId
    * @param userPw
    * @param data
    */
   public static void clearFromDatabase(String address, String userId, String userPw, ArrayList<String> data) 
   {
      scriptAddress = address;


      try 
      {
         url = new URL(scriptAddress); conn = url.openConnection(); conn.setDoOutput(true); 
      

         dataSend = URLEncoder.encode("userId", "UTF-8") + "=" + URLEncoder.encode(userId, "UTF-8")

         + "&" + URLEncoder.encode("userPw", "UTF-8") + "=" + URLEncoder.encode(userPw, "UTF-8");
         
         
         if (data != null) {setVariableValues(data);} sendData();
         
         reader = new BufferedReader(new InputStreamReader(conn.getInputStream()));


         while ((dataRecieve = reader.readLine()) != null) 
         {

            System.out.println(dataRecieve);
         }


         reader.close();
         
      }
      
      catch (IOException e) {System.out.println(Tools.thisPathAndLine() + e + "\n");}

   }
   
   
   //***************************************** END PUBLIC METHODS ******************************************//


   //**************************************** START PRIVATE METHODS ****************************************//

   
   /**
    * Sets the PHP variable values.
    * ESCA-JAVA0266:
    * @param data
    */
   private static void setVariableValues(ArrayList<String> data)
   {
      try
      {
         for (int i = 0; i < data.size(); i ++)
         {
            String variableName = "variable_" + Integer.toString(i+1);
       
            dataSend += "&" + URLEncoder.encode(variableName, "UTF-8") +

            "=" + URLEncoder.encode(data.get(i), "UTF-8");
         } 
      }
      
      catch (UnsupportedEncodingException e) {System.out.println(Tools.thisPathAndLine() + e + "\n");}
   }

   
   
   /**
    * Sends data to the appropriate PHP script.
    * ESCA-JAVA0266:
    */
   private static void sendData()
   {
      try
      {    
         writer = new BufferedWriter(new OutputStreamWriter(conn.getOutputStream()));
  
         writer.write(dataSend); writer.flush(); writer.close();
      }
 
      catch (MalformedURLException e) {System.out.println(Tools.thisPathAndLine() + e + "\n");}

      catch (IOException e) {System.out.println(Tools.thisPathAndLine() + e + "\n");}
   
   }


   //***************************************** END PRIVATE METHODS *****************************************//
   
   
   private static URL url;
   private static String dataSend;
   private static URLConnection conn;
   private static String dataRecieve;
   private static String scriptAddress;
   private static BufferedReader reader;
   private static BufferedWriter writer; 
}


//********************************************** END CLASS **********************************************//